/*
 * Messages for OpenMoko
 * Copyright (C) 2007 Milko Krachounov <exabyte@3mhz.net>
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */



#include <gtk/gtk.h>
#include <string.h>

//#include <moko-stock.h>

#include "messages-defines.h"
#include "messages-common.h"
#include "messages-access.h"


enum {
	FOLDER_TYPE_BUILTIN = 1,
	FOLDER_TYPE_LOCAL,
	FOLDER_TYPE_IMAP
};


typedef struct {
	gchararray account;
	gchararray path;
} imap_folder_info;

typedef struct {
	gchararray path;
} local_folder_info;


typedef struct {
	gint id;
	gint type;
	union {
		imap_folder_info imap;
		local_folder_info local;
	} storage;
} folder_info;

void messages_access_init(gboolean (*cb) (const event_info*, gpointer), gpointer data)
{
	return;
}

gboolean
get_message_list(gint folder_id, gboolean (*cb) (const message_list_info*, gpointer), gpointer data)
{
	message_list_info message;
	gboolean result = TRUE;

	switch (folder_id) {
		case FOLDER_INBOX:
			/* Some example messages go here */
			message.type = MESSAGE_TYPE_SMS;
			message.id = 1; 
			message.ts = 1184113787;
			message.subject = "Test message 1";
			message.tofrom = "+1234567890";
			message.flags = MESSAGE_FLAG_UNREAD;
		
			result &= cb(&message, data);

			message.type = MESSAGE_TYPE_SMS;
			message.id = 2;
			message.ts = 1194113787;
			message.subject = "Test message 2";
			message.tofrom = "+1234567890";
			message.flags = MESSAGE_FLAG_UNREAD;

			result &= cb(&message, data);
			

			message.type = MESSAGE_TYPE_EMAIL;
			message.id = 1;
			message.ts = 1194112787;
			message.subject = "Mail message";
			message.tofrom = "Internation Тест";
			message.flags = MESSAGE_FLAG_UNREAD;

			result &= cb(&message, data);
			/*
			message = { MESSAGE_TYPE_SMS,
				    2,
				    1194113787,
				    "Test message 2",
				    "1234567890",
				    MESSAGE_FLAG_UNREAD };
			*/	
			/*
			gtk_list_store_append (GTK_LIST_STORE(app_data->message_list), &iter);
			gtk_list_store_set (GTK_LIST_STORE(app_data->message_list), &iter,
					    COL_TYPE, MESSAGE_TYPE_SMS,
					    COL_ID, 2,
					    COL_TOFROM, "+1234567890",
					    COL_SUBJECT, "Test message 2",
					    COL_ICON, "mail-message-new",
					    COL_TS, 1194113787,
					    COL_UNREAD, 0,
					    -1);
			*/
		break;
	}


	return result;
}



gboolean
get_folder_list(gint parent, gboolean (*cb) (const folder_list_info*, gpointer), gboolean recursive, gpointer data)
{
	folder_list_info folder;
	gboolean status = TRUE;
	if (parent == FOLDER_ROOT) {
		folder.parent_id = FOLDER_ROOT;

		folder.flags = FOLDER_FLAG_DEFAULT;
	
		folder.id = FOLDER_INBOX;
		folder.name = "Inbox";
		status &= cb(&folder, data);

		folder.flags |= FOLDER_FLAG_OUTGOING;
		
		folder.id = FOLDER_OUTBOX;
		folder.name = "Outbox";
		status &= cb(&folder, data);
		
		folder.id = FOLDER_SENT;
		folder.name = "Sent";
		status &= cb(&folder, data);
#ifndef DEBUG_NOFOLDERS		

		folder.flags |= FOLDER_FLAG_DRAFTS;
		folder.id = FOLDER_DRAFTS;
		folder.name = "Drafts";
		status &= cb(&folder, data);
		folder.flags &= ~FOLDER_FLAG_DRAFTS;
		
		folder.id = FOLDER_TEMPLATES;
		folder.name = "Templates";
		status &= cb(&folder, data);

		
		folder.flags = FOLDER_FLAG_NO_SUBFOLDERS;
		folder.id = FOLDER_SEARCH;
		folder.name = "Search";
		status &= cb(&folder, data);
#endif		

	}
	return status;
}



gboolean message_get_sms(gint folder_id, gint id, sms_display_info *value)
{
	if (folder_id != FOLDER_INBOX || id < 1 || id > 2)
		return FALSE;
	/* A message from my imaginary girlfriend */
	value->message = gtk_text_buffer_new(NULL);
	gtk_text_buffer_set_text(value->message,
        "Hello, my dear. This is my message for you!", -1);
	value->tofrom = g_strdup("+1234567890");
	value->ts = 1184113787;
	value->flags = MESSAGE_FLAG_DEFAULT;
	return TRUE;
}


gboolean message_get_mail(gint folder_id, gint id, mail_display_info *value)
{
	mail_attachment *attach;

	if (folder_id != FOLDER_INBOX || id != 1)
		return FALSE;
	/* A message about my imaginary journey */
	value->message = gtk_text_buffer_new(NULL);;
	gtk_text_buffer_set_text(value->message,
	"Hello. We are writing you in connection with your journey to Mars. "
	"We like to inform you that everything has been settled, and "
	"we'll be requesting you additional infomation in a week.\n\n"
	"We hope that you will be departing in June.\n\n"
	"With best regards,\nJohn Interstellar", -1);
	value->from = g_strdup("Interstellar <interstellar@example.com>");
	value->to = g_strdup("Me <me@example.com>");
	value->cc = NULL;
	value->bcc = NULL;
	value->ts = 1194112787;
	value->subject = g_strdup("Your journey details are settled.");
	value->attachments_size = 0;
	value->attachments = NULL;

	attach = g_malloc(sizeof(mail_attachment));
	attach->id = 0;
	attach->name = g_strdup("details.odt");
	attach->size = 28318;
	attach->path = NULL;
	attach->folder_id = folder_id;
	attach->message_id = id;
	value->attachments_size += attach->size;
	value->attachments = g_list_append(value->attachments, attach);

	attach = g_malloc(sizeof(mail_attachment));
	attach->id = 1;
	attach->name = g_strdup("Your Little Science Software Program.tar.gz");
	attach->size = 3827182;
	attach->path = NULL;
	attach->folder_id = folder_id;
	attach->message_id = id;
	value->attachments_size += attach->size;
	value->n_attachments = 2;
	value->attachments = g_list_append(value->attachments, attach);


	return TRUE;
}



gboolean message_delete(gint folder_id, gint type, gint id)
{
	return TRUE;
}
gboolean message_send_sms(const sms_display_info *message, message_list_info *outbox_entry)
{
	return FALSE;
}
gboolean message_send_mail(const mail_display_info *message, message_list_info *outbox_entry)
{
	return FALSE;
}
gboolean message_move(gint src_fid, gint src_id, gint dst_fid, gint dst_id)
{
	return FALSE;
}


gboolean message_mail_save_attachment(gint folder_id, gint mail_id, gint id, const gchar *path)
{
	return FALSE;
}

void display_info_unset(display_info *p)
{
	switch (p->type) {
		case MESSAGE_TYPE_SMS:
			sms_display_info_unset(&(p->info.sms));
			break;
		#if 0
		case MESSAGE_TYPE_MMS:
			mms_display_info_unset(&(p->info.mms));
			break;
		#endif
		case MESSAGE_TYPE_EMAIL:
			mail_display_info_unset(&(p->info.mail));
			break;
	}
}

void sms_display_info_unset(sms_display_info *p)
{
	if (p->message)
		g_object_unref(p->message);
	if (p->tofrom)
		g_free(p->tofrom);
}
#if 0
void mms_display_info_unset(mms_display_info *p)
{
}
#endif

static void attachment_unset (mail_attachment *p, gpointer data)
{
	if (p) {
		if (p->name)
			g_free(p->name);
		if (p->path)
			g_free(p->path);
		g_free(p);
	}
}

void mail_display_info_unset(mail_display_info *p)
{
	if (p->message)
		g_object_unref(p->message);
	if (p->from)
		g_free(p->from);
	if (p->to)
		g_free(p->to);
	if (p->cc)
		g_free(p->cc);
	if (p->bcc)
		g_free(p->bcc);
	if (p->subject)
		g_free(p->subject);
	if (p->attachments) {
		g_list_foreach(p->attachments, (GFunc) attachment_unset, NULL);
		g_list_free(p->attachments);
	}
}


gboolean folder_is_online(gint id)
{
	return TRUE;
}

gboolean folder_can_be_renamed(gint id)
{
	return FALSE;
}
