/*
 * Messages for OpenMoko
 * Copyright (C) 2007 Milko Krachounov <exabyte@3mhz.net>
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include <gtk/gtk.h>
#include <strings.h>
#include <glib/gi18n.h>

#include "messages-defines.h"
#include "messages-common.h"
#include "messages-access.h"
#include "menutoolbutton.h"
#include "mokofieldlabel.h"


void init_app_data(AppData *app_data)
{
	/* We need no actual initialisation, beside zeroing. */
	bzero(app_data, sizeof(AppData));
}


gint
notebook_add_page_with_icon (GtkNotebook *notebook, GtkWidget *child, const gchar *icon_name, int padding)
{
	gint page_num;
	GtkWidget *icon = gtk_image_new_from_icon_name (icon_name, GTK_ICON_SIZE_LARGE_TOOLBAR);
	GtkWidget *align = gtk_alignment_new (0.5, 0.5, 1.0, 1.0);
	gtk_alignment_set_padding (GTK_ALIGNMENT (align), padding, padding, padding, padding);
	gtk_container_add (GTK_CONTAINER (align), icon);
	gtk_widget_show_all (align);
	page_num = gtk_notebook_append_page (notebook, child, align);
	gtk_container_child_set (GTK_CONTAINER (notebook), child, "tab-expand", TRUE, NULL);
	return page_num;
}


void 
toolbar_insert_menu_button (GtkToolbar* toolbar, GtkWidget *icon, GCallback callback, gpointer data, GtkWidget* menu)
{
	
	GtkToolItem *button = moko_menu_tool_button_new(icon, "");
	moko_menu_tool_button_set_menu  (MOKO_MENU_TOOL_BUTTON(button), menu);
	g_signal_connect (G_OBJECT (button), "clicked", callback, data);
	gtk_tool_item_set_expand (button, TRUE);
	if (gtk_toolbar_get_n_items(GTK_TOOLBAR(toolbar)))
		gtk_toolbar_insert (GTK_TOOLBAR (toolbar), gtk_separator_tool_item_new (), -1);

	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), button, -1);	
}


void 
toolbar_insert_button (GtkToolbar* toolbar, GtkWidget *icon, GCallback callback, gpointer data)
{
	
	GtkToolItem *button = gtk_tool_button_new(icon, "");
	g_signal_connect (G_OBJECT (button), "clicked", callback, data);
	gtk_tool_item_set_expand (button, TRUE);
	if (gtk_toolbar_get_n_items(GTK_TOOLBAR(toolbar)))
		gtk_toolbar_insert (GTK_TOOLBAR (toolbar), gtk_separator_tool_item_new (), -1);

	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), button, -1);	
}

#if 0
void
toolbar_insert_button_from_stock (GtkWidget *toolbar, const gchar *icon_name, GCallback callback, gpointer data)
{
	GtkToolItem *button = gtk_tool_button_new_from_stock (icon_name);
	g_signal_connect (G_OBJECT (button), "clicked", callback, data);
	
	gtk_tool_item_set_expand (button, TRUE);
	if (gtk_toolbar_get_n_items(GTK_TOOLBAR(toolbar)))
		gtk_toolbar_insert (GTK_TOOLBAR (toolbar), gtk_separator_tool_item_new (), 0);

	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), button, 0);

}
#endif


/* We need a function to remove all widgets from a container */
/* Callback used for mass removal of widgets from a container */
static void
cb_widget_remove (GtkWidget* widget, GtkContainer *container)
{
	gtk_container_remove(container, widget);
}

void
container_remove_all_widgets (GtkContainer *container)
{
	gtk_container_foreach (container, (GtkCallback)cb_widget_remove, container);
}

/* Getting values from a GtkTreeModel */
gint
tree_get_value_int (GtkTreeModel *tree_model, GtkTreeIter *iter, gint column)
{
	GValue value = {0, };
	gint result;
	
	gtk_tree_model_get_value(tree_model, iter, column, &value);
	result = g_value_get_int(&value);
	g_value_unset(&value);

	return result;
}

/* NOTE: Return value should be freed */
gchararray
tree_get_value_string (GtkTreeModel *tree_model, GtkTreeIter *iter, gint column)
{
	GValue value = {0, };
	gchararray result;
	
	gtk_tree_model_get_value(tree_model, iter, column, &value);
	result = g_value_dup_string(&value);
	g_value_unset(&value);

	return result;
}

/* TODO: Move to access */
gchararray
get_contact_name (gchararray number)
{
	/* TODO: Implement me */
	return number;
}

/* A dialog which seeks the user input to delete a message */
gboolean
message_delete_dialog (gint folder, gint type, gint id)
{
	GtkWidget *dialog;
	gint dialog_result;

	dialog = gtk_message_dialog_new(0, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO, _("Delete this %s?"), type == MESSAGE_TYPE_SMS ? _("SMS") : _("mail"));
	gtk_window_set_title(GTK_WINDOW(dialog), " ");

	dialog_result = gtk_dialog_run(GTK_DIALOG(dialog));
	gtk_widget_destroy(dialog);

	if (dialog_result == GTK_RESPONSE_YES) {
		if (!message_delete(folder, type, id)) {
			dialog = gtk_message_dialog_new(0, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, _("Deleting the message failed."));
			gtk_dialog_run(GTK_DIALOG(dialog));
			gtk_widget_destroy(dialog);
			return FALSE;
		}
		return TRUE;
	}
	return FALSE;
}

/* We need to convert file sizes to human readable format */
/* This should be in order, and the last should be NULL, -1 */
static SizeUnit binary_units[] = {	
	{ N_("B"), 1 },
	{ N_("KB"), 1024 },
	{ N_("MB"), 1048576 },
	{ N_("GB"), 1073741824 },
	{ NULL, -1 }
};

/* 
 *  This function returns the suitable factor, which we should divide the size
 *  by and sets *unit to the respective units used
 */

off_t
get_size_display_factor(off_t size, const gchar **unit)
{
	gint i;
	off_t factor;
	const gchar* name;
	i = 0;
	do {
		name = binary_units[i].name;
		factor = binary_units[i].size;
	} while (size / binary_units[++i].size >= 1);

	if (unit)
		*unit = name;
	return factor;
}

/* Field table creation */
struct widget_pair {
	GtkWidget *label;
	GtkWidget *value;
};

GList* header_table_append_label(GList *row_list, 
				 const gchar *label,
				 const gchar *value)
{
	struct widget_pair *row = g_slice_alloc(sizeof(struct widget_pair));

	row->label = gtk_label_new(label);
	row->value = moko_field_label_new(value);

	gtk_label_set_selectable (GTK_LABEL(row->value), TRUE);

	return g_list_append(row_list, row);
}

	
GList* header_table_append_entry(GList *row_list, 
				 const gchar *label, 
				 GtkWidget **entry,
				 const char *value)
{
	struct widget_pair *row = g_slice_alloc(sizeof(struct widget_pair));

	row->label = gtk_label_new(label);
	row->value = gtk_entry_new();

	gtk_entry_set_text(GTK_ENTRY(row->value), value);
	
	*entry = row->value;

	return g_list_append(row_list, row);
}

static void
header_table_row_free(struct widget_pair *row, gpointer size)
{
	g_slice_free1(GPOINTER_TO_SIZE(size), row);
}

GtkWidget*
header_table_create(GList *row_list, gfloat yalign, gint ypadding)
{

	GtkWidget *table;
	PangoAttrList *bold;
	PangoAttribute *attr;
	GList *list_elem;
	struct widget_pair *row;
	gint rows, i;

	
	/* Bold attribute */
	bold = pango_attr_list_new();	
	attr = pango_attr_weight_new(PANGO_WEIGHT_BOLD);
	attr->start_index = 0;
	attr->end_index = G_MAXUINT;
	pango_attr_list_insert(bold, attr);
	
	/* Count rows */
	rows = g_list_length (row_list);

	/* Table initialisation */
	table = gtk_table_new (rows, 2, FALSE);
	gtk_table_set_row_spacings(GTK_TABLE(table), 0);
	gtk_table_set_col_spacings(GTK_TABLE(table), 0);
	
	
	for (list_elem = row_list, i = 0;
	     list_elem;
	     list_elem = g_list_next(list_elem), i++)
	{
		row = (struct widget_pair *) list_elem->data;

		gtk_label_set_attributes(GTK_LABEL(row->label), bold);
		gtk_misc_set_alignment(GTK_MISC(row->label), 1.0f, yalign);
		gtk_table_attach(GTK_TABLE(table), row->label, 0, 1, i, i+1, GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_EXPAND, 3, ypadding);
	

		gtk_table_attach(GTK_TABLE(table), row->value, 1, 2, i, i+1, GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, 3, ypadding);
		gtk_misc_set_alignment(GTK_MISC(row->value), 0.0f, yalign);
		
		
	}

	/* Free everything */
	/* gsize and gpointer should be the same size on every platform */
	g_list_foreach(row_list, (GFunc) header_table_row_free, GSIZE_TO_POINTER(sizeof(struct widget_pair)));
	pango_attr_list_unref(bold);

	return table;
}

