/*
 * Messages for OpenMoko
 * Copyright (C) 2007 Milko Krachounov <exabyte@3mhz.net>
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */


#ifndef MESSAGES_COMMON_H
#define MESSAGES_COMMON_H
#include "messages-access.h"
#include <sys/types.h>

/* Struct for application-wide data */
typedef struct {	
	GtkWindow *window;

	GtkNotebook *notebook;

	GtkTreeModel *message_list;
	GtkTreeView *message_view;
	GtkTreeIter *current_message;
	GtkTreeIter *last_message;
	
	GtkTreeModel *folder_list;
	GtkTreeIter *current_folder;
	GtkTreeIter *last_folder;
	
	display_info selected;

	GtkWidget *display_pane;

	GtkPaned *paned;
	
} AppData;

typedef struct {
	const gchar* name;
	off_t size;
} SizeUnit;

void init_app_data(AppData *app_data);

gint
notebook_add_page_with_icon (GtkNotebook *notebook,
			     GtkWidget *child,
			     const gchar *icon,
			     gint padding);
void
toolbar_insert_button (GtkToolbar*, GtkWidget *icon, GCallback, gpointer data);
void
toolbar_insert_menu_button (GtkToolbar*, GtkWidget *icon, GCallback, gpointer data, GtkWidget* menu); 

void container_remove_all_widgets (GtkContainer *container);

gint	   tree_get_value_int    (GtkTreeModel *, GtkTreeIter *, gint column);
gchararray tree_get_value_string (GtkTreeModel *, GtkTreeIter *, gint column);

gchararray	get_contact_name (gchararray number);

gboolean	message_delete_dialog (gint folder, gint type, gint id);

off_t		get_size_display_factor(off_t size, const gchar **unit);

/*
 * Functions used for creating the header display and input. 
 * It has two functions which create a GList of the values or inputs, and a function
 * that returns the table widget
 *
 * When appending an entry value, the function takes a pointer to a GtkWidget*,
 * which will be changed to point to the entry widget.
 *
 * At creation, the allocated memory is freed.
 */

GList* header_table_append_label(GList *row_list, 
				 const gchar *field, 
				 const gchar *value) G_GNUC_WARN_UNUSED_RESULT;
GList* header_table_append_entry(GList *row_list, 
				 const gchar *field, 
				 GtkWidget **entry,
				 const char *value) G_GNUC_WARN_UNUSED_RESULT;

GtkWidget *header_table_create(GList *row_list, gfloat yalign, gint padding);


#endif
