/*
 * Messages for OpenMoko
 * Copyright (C) 2007 Milko Krachounov <exabyte@3mhz.net>
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include <gtk/gtk.h>
#include <glib/gi18n.h>
#include <moko-finger-scroll.h>
#include <moko-stock.h>
#include <strings.h>

#include "mokofieldlabel.h"
#include "messages-defines.h"
#include "messages-common.h"
#include "messages-access.h"
#include "messages-list.h"
#include "messages-read.h"


typedef struct {
	AppData *app_data;
	display_info message;
	GtkWidget *page;
} MessagesReadPageData;


/* SMS functions */
/* This displays the message in a GtkBox container. The container MUST be empty before running. */
void
message_sms_display (GtkBox* container, display_info *what)
{
	
	sms_display_info *message = &(what->info.sms);
	
	GtkWidget *table;
	GtkWidget *text;
	GList *row_list = NULL;
	
	time_t message_ts;
	struct tm tmp;

	/* Do not reduce this, it is possible that the date is more bytes
           in some locales, while I highly doubt it, 128 is not that much. */
	gchar buf[128];

	/* We get the message */
	if (message_get_sms(what->folder, what->id, message)) {
	
		row_list = header_table_append_label(
				row_list,
				message->flags & MESSAGE_FLAG_OUTGOING ? _("To:") : _("From:"),
				message->tofrom	
		);
		
		/* We get the localised time string */
		message_ts = message->ts;
		localtime_r(&message_ts, &tmp);
		strftime(buf, sizeof(buf), "%c", &tmp);

		row_list = header_table_append_label(row_list, _("Date:"), buf);
		
		table = header_table_create(row_list, 0.0f, 3);
		gtk_box_pack_start (GTK_BOX (container), table, FALSE, TRUE, 0);
	
		text = gtk_text_view_new_with_buffer(message->message);
		/* FIXME: Text can't be currently selected at all  */
		//g_object_set(G_OBJECT(text), "can-focus", FALSE, NULL);
		/* We need this to enable scrolling and to get rid of the keyboard */
		gtk_text_view_set_editable(GTK_TEXT_VIEW(text), FALSE);
		gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW(text), FALSE);
		gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(text), GTK_WRAP_WORD_CHAR);
		gtk_widget_set_sensitive(GTK_WIDGET(text), FALSE);
		gtk_box_pack_start (GTK_BOX (container), text, TRUE, TRUE, 0);
	}
	else {
		text = gtk_label_new(_("Could not find message"));
		gtk_box_pack_start (GTK_BOX (container), text, TRUE, TRUE, 0);
	}
	gtk_widget_show_all (GTK_WIDGET(container));

}

/* Mail functions */
static void
cb_attachment_open(GtkButton *button, mail_attachment *attachment)
{
	
	/* Just in case something happens to our GList... */
	gint id = attachment->id;
	gint folder_id = attachment->folder_id;
	gint mail_id = attachment->message_id;
	GtkWidget *dialog;
	gint dialog_result;

	/* TODO: Implement folder chooser */
	const gchar *path = "/media/card/";

	dialog = gtk_message_dialog_new(0, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO, _("Do you want to save the attachment %s in %s?"), attachment->name, path);
	gtk_window_set_title(GTK_WINDOW(dialog), " ");

	dialog_result = gtk_dialog_run(GTK_DIALOG(dialog));
	gtk_widget_destroy(dialog);

	if (dialog_result == GTK_RESPONSE_YES) {
		if (!message_mail_save_attachment(folder_id, mail_id, id, path)) {
			dialog = gtk_message_dialog_new(0, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, _("Saving the attachment failed."));
			gtk_dialog_run(GTK_DIALOG(dialog));
			gtk_widget_destroy(dialog);
		}
	}

}
static void
display_attachment (mail_attachment *attachment, GtkWidget *vbox)
{
	char buf[32];
	const gchar *unit;
	GtkWidget *hbox;
	GtkWidget *name, *size;
	GtkWidget *button;
	PangoAttrList *bold;
	PangoAttribute *attr;

	/* Bold attribute */
	bold = pango_attr_list_new();	
	attr = pango_attr_weight_new(PANGO_WEIGHT_BOLD);
	attr->start_index = 0;
	attr->end_index = G_MAXUINT;
	pango_attr_list_insert(bold, attr);

	hbox = gtk_hbox_new(FALSE, 3);
	
	name = moko_field_label_new(attachment->name);
	gtk_label_set_ellipsize(GTK_LABEL(name), PANGO_ELLIPSIZE_END);
	gtk_label_set_attributes(GTK_LABEL(name), bold);

	button = gtk_button_new();
	g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(cb_attachment_open), attachment);

	snprintf(buf, sizeof(buf), "%.2f %s", attachment->size / (double)get_size_display_factor(attachment->size, &unit), unit);
	size = gtk_label_new(buf);

	gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, TRUE, 0);
	gtk_box_pack_start (GTK_BOX (hbox), name, TRUE, TRUE, 0);
	gtk_box_pack_start (GTK_BOX (hbox), size, FALSE, TRUE, 0);
	
	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, TRUE, 0);

	
	
}

void
message_email_display (GtkBox* container, display_info *what)
{
	mail_display_info *message = &(what->info.mail);
	
	GtkWidget *table;
	GtkWidget *text;
	GtkWidget *expander, *vbox;
	const gchar *unit_name;
	GList *row_list = NULL;

	time_t message_ts;
	struct tm tmp;
	/* Buffer used for date and other stuff.
	   Do not reduce the size, it is possible that the date is more bytes
           in some locales, while I highly doubt it, 128 is not that much. */

	gchar buf[128];

	/* We get the message */
	if (message_get_mail(what->folder, what->id, message)) {
	

	
		row_list = header_table_append_label(row_list, _("From:"), message->from);
		
		if (message->subject) {
			row_list = header_table_append_label(row_list, _("Subject:"), message->subject);
		}
	
		/* We get the localised time string */
		message_ts = message->ts;
		localtime_r(&message_ts, &tmp);
		strftime(buf, sizeof(buf), "%c", &tmp);

		row_list = header_table_append_label(row_list, _("Date:"), buf);
	

		row_list = header_table_append_label(row_list, _("To:"), message->to);
	
	
		if (message->cc) {
			row_list = header_table_append_label(row_list, _("Cc:"), message->cc);
		}
	
		if (message->bcc) {
			row_list = header_table_append_label(row_list, _("Bcc:"), message->bcc);
		}
	
		table = header_table_create(row_list, 0.0f, 3);
		gtk_box_pack_start (GTK_BOX (container), table, FALSE, TRUE, 0);
	
		text = gtk_text_view_new_with_buffer(message->message);

		/* Display the attachments */
		if (message->n_attachments > 0) {
			/* Hope the translators would guess that %d %s is number word :D */
			snprintf(buf, sizeof(buf), 
				ngettext("%d attachment (%.2f %s)", "%d attachments (%.2f %s)", message->n_attachments),
				message->n_attachments,
				message->attachments_size / (double)get_size_display_factor(message->attachments_size, &unit_name),
				_(unit_name));
			expander = gtk_expander_new(buf);
			vbox = gtk_vbox_new(FALSE, 0);

			gtk_expander_set_spacing (GTK_EXPANDER(expander), 2);
			gtk_container_set_border_width(GTK_CONTAINER(expander), 5);
			gtk_container_add(GTK_CONTAINER(expander), vbox);
			gtk_box_pack_start(GTK_BOX(container), expander, FALSE, TRUE, 0);
		
			g_list_foreach(message->attachments, (GFunc) display_attachment, vbox);
			
			
	
		}

		/* FIXME: Text can't be currently selected at all  */
		//g_object_set(G_OBJECT(text), "can-focus", FALSE, NULL);
		/* We need this to enable scrolling and to get rid of the keyboard */
		gtk_text_view_set_editable(GTK_TEXT_VIEW(text), FALSE);
		gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW(text), FALSE);
		gtk_widget_set_sensitive(GTK_WIDGET(text), FALSE);

		gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(text), GTK_WRAP_WORD_CHAR);
		gtk_box_pack_start (GTK_BOX (container), text, TRUE, TRUE, 0);
		
	
	}
	else {
		text = gtk_label_new(_("Could not find message"));
		gtk_box_pack_start (GTK_BOX (container), text, TRUE, TRUE, 0);
	}
	gtk_widget_show_all (GTK_WIDGET(container));
}

static void
cb_message_close (GtkToolButton *button, MessagesReadPageData *data)
{
	gint page_num;
	page_num = gtk_notebook_page_num(data->app_data->notebook, data->page);
	gtk_notebook_remove_page(data->app_data->notebook, page_num);
	display_info_unset(&(data->message));
	g_free(data);
}

static void
cb_message_reply_all (GtkToolButton *button, MessagesReadPageData *data)
{

}

static void
cb_message_forward (GtkToolButton *button, MessagesReadPageData *data)
{

}

static void
cb_message_delete (GtkToolButton *button, MessagesReadPageData *data)
{
	if (message_delete_dialog (data->message.folder, data->message.type, data->message.id)) {
		list_delete_message(data->message.folder, data->message.type, data->message.id, data->app_data);
		cb_message_close(button, data);
	}
}

GtkWidget*
messages_create_read_page (AppData* app_data)
{
	GtkWidget *page, *toolbar, *viewport, *scroll, *message;

	gint folder = app_data->selected.folder;
	gint type = app_data->selected.type;
	gint id = app_data->selected.id;

	MessagesReadPageData *data = g_malloc(sizeof(MessagesReadPageData));
	bzero(data, sizeof(MessagesReadPageData));

	
		
	page = gtk_vbox_new (FALSE, 0);

	/* We create the toolbar */
	toolbar = gtk_toolbar_new ();

	toolbar_insert_button (GTK_TOOLBAR(toolbar), gtk_image_new_from_icon_name (GTK_STOCK_CLOSE, GTK_ICON_SIZE_LARGE_TOOLBAR), G_CALLBACK(cb_message_close), data);
	toolbar_insert_button (GTK_TOOLBAR(toolbar), gtk_image_new_from_icon_name (MOKO_STOCK_MAIL_REPLY_ALL, GTK_ICON_SIZE_LARGE_TOOLBAR), G_CALLBACK(cb_message_reply_all), data);
	toolbar_insert_button (GTK_TOOLBAR(toolbar), gtk_image_new_from_icon_name (MOKO_STOCK_MAIL_FORWARD, GTK_ICON_SIZE_LARGE_TOOLBAR), G_CALLBACK(cb_message_forward), data);
	toolbar_insert_button (GTK_TOOLBAR(toolbar), gtk_image_new_from_icon_name (MOKO_STOCK_MAIL_DELETE, GTK_ICON_SIZE_LARGE_TOOLBAR), G_CALLBACK(cb_message_delete), data);

	gtk_box_pack_start (GTK_BOX (page), toolbar, FALSE, TRUE, 0);

	message = gtk_vbox_new (FALSE, 0);
	viewport = gtk_viewport_new(NULL, NULL);
	gtk_viewport_set_shadow_type (GTK_VIEWPORT (viewport), GTK_SHADOW_NONE);

	scroll = moko_finger_scroll_new();
	gtk_container_add (GTK_CONTAINER (viewport), message);	
	gtk_container_add (GTK_CONTAINER (scroll), viewport);
	
	gtk_box_pack_end (GTK_BOX (page), scroll, TRUE, TRUE, 0);


	data->message.folder = folder;
	data->message.type = type;
	data->message.id = id;

	if (type == MESSAGE_TYPE_SMS) {
		message_sms_display(GTK_BOX(message), &(data->message));
	}
	else if (type == MESSAGE_TYPE_EMAIL) {
		message_email_display(GTK_BOX(message), &(data->message));
	}
	data->app_data = app_data;
	data->page = page;


	return page;

}
