/*
 * Mail for OpenMoko
 * Copyright (C) 2007 Milko Krachounov <exabyte@3mhz.net>
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include <gtk/gtk.h>
#include <glib/gi18n.h>
#include <strings.h>
#include <moko-finger-scroll.h>
#include <moko-stock.h>

#include "mail-defines.h"
#include "mail-common.h"
#include "mail-write.h"



typedef struct {
	AppData *app_data;
	display_info message;
	GtkWidget *page;
	union {
		struct {
			GtkWidget *tofrom;
		} sms;
		struct {
			GtkWidget *to;
			GtkWidget *subject;
			GtkWidget *cc;
			GtkWidget *bcc;
		} email;
			
	} entries;
} MessagesWritePageData;

static void
cb_message_close (GtkToolButton *button, MessagesWritePageData *data)
{
	gint page_num;
	page_num = gtk_notebook_page_num(data->app_data->notebook, data->page);
	gtk_notebook_remove_page(data->app_data->notebook, page_num);
	g_free(data);
}

static void
cb_send_sms (GtkToolButton *button, MessagesWritePageData *data)
{

}

static void
cb_send_email (GtkToolButton *button, MessagesWritePageData *data)
{

}

GtkWidget*
messages_create_write_sms_page (AppData* app_data)
{
	GtkWidget *page, *toolbar, *viewport, *scroll, *table, *message, *text;
	GList *row_list = NULL;

	MessagesWritePageData *data = g_malloc(sizeof(MessagesWritePageData));
	bzero(data, sizeof(MessagesWritePageData));

	page = gtk_vbox_new (FALSE, 0);

	toolbar = gtk_toolbar_new ();
	toolbar_insert_button (GTK_TOOLBAR(toolbar), gtk_image_new_from_icon_name (GTK_STOCK_CLOSE, GTK_ICON_SIZE_LARGE_TOOLBAR), G_CALLBACK(cb_message_close), data);
	toolbar_insert_button (GTK_TOOLBAR(toolbar), gtk_image_new_from_icon_name (MOKO_STOCK_MAIL_SEND, GTK_ICON_SIZE_LARGE_TOOLBAR), G_CALLBACK(cb_send_sms), data);
	
	gtk_box_pack_start (GTK_BOX (page), toolbar, FALSE, TRUE, 0);


	/* This is our input */
	row_list = header_table_append_entry(row_list, _("To:"), &(data->entries.sms.tofrom), data->message.info.sms.tofrom);
	table = header_table_create(row_list, 0.5f, 3);
	gtk_box_pack_start (GTK_BOX (page), table, FALSE, TRUE, 0);

	viewport = gtk_viewport_new(NULL, NULL);
	gtk_viewport_set_shadow_type (GTK_VIEWPORT (viewport), GTK_SHADOW_NONE);

	//message = gtk_vbox_new (FALSE, 0);
	if (!data->message.info.sms.message)
		data->message.info.sms.message = gtk_text_buffer_new(NULL);
	text = gtk_text_view_new_with_buffer(data->message.info.sms.message);
	gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(text), GTK_WRAP_WORD_CHAR);
	gtk_container_set_border_width(GTK_CONTAINER(text), 1);

	//gtk_box_pack_start (GTK_BOX (message), text, TRUE, TRUE, 0);
	
	//viewport = gtk_viewport_new(NULL, NULL);
	

	scroll = gtk_scrolled_window_new (NULL, NULL);
	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scroll), GTK_SHADOW_ETCHED_IN);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scroll), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);

	//gtk_container_add (GTK_CONTAINER (viewport), text);	
	gtk_container_add (GTK_CONTAINER (scroll), text);
	

	gtk_box_pack_end (GTK_BOX (page), scroll, TRUE, TRUE, 0);

	data->app_data = app_data;
	data->page = page;
	
	return page;

	

}
GtkWidget*
messages_create_write_email_page (AppData* app_data)
{
	
	GtkWidget *page, *toolbar, *scroll, *table, *text;
	GtkWidget *notebook;
	GList *row_list;
	
	MessagesWritePageData *data = g_malloc(sizeof(MessagesWritePageData));
	bzero(data, sizeof(MessagesWritePageData));

	page = gtk_vbox_new (FALSE, 0);

	toolbar = gtk_toolbar_new ();
	toolbar_insert_button (GTK_TOOLBAR(toolbar), gtk_image_new_from_icon_name (GTK_STOCK_CLOSE, GTK_ICON_SIZE_LARGE_TOOLBAR), G_CALLBACK(cb_message_close), data);
	toolbar_insert_button (GTK_TOOLBAR(toolbar), gtk_image_new_from_icon_name (MOKO_STOCK_MAIL_SEND, GTK_ICON_SIZE_LARGE_TOOLBAR), G_CALLBACK(cb_send_email), data);
	
	gtk_box_pack_start (GTK_BOX (page), toolbar, FALSE, TRUE, 0);

	/* We create a vertical notebook with the different input fields */	

	notebook = gtk_notebook_new();
	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(notebook), GTK_POS_LEFT);

	row_list = NULL;
	row_list = header_table_append_entry(row_list, _("To:"), &(data->entries.email.to), data->message.info.mail.to);
	row_list = header_table_append_entry(row_list, _("Subject:"), &(data->entries.email.subject), data->message.info.mail.subject);
	table = header_table_create(row_list, 0.5f, 1);
	gtk_notebook_append_page (GTK_NOTEBOOK(notebook), table, gtk_label_new("O"));
	
	row_list = NULL;
	row_list = header_table_append_entry(row_list, _("Cc:"), &(data->entries.email.cc), data->message.info.mail.cc);
	row_list = header_table_append_entry(row_list, _("Bcc:"), &(data->entries.email.bcc), data->message.info.mail.bcc);
	table = header_table_create(row_list, 0.5f, 3);
	gtk_notebook_append_page (GTK_NOTEBOOK(notebook), table, gtk_label_new("O"));
	gtk_box_pack_start (GTK_BOX (page), notebook, FALSE, TRUE, 0);

	//message = gtk_vbox_new (FALSE, 0);
	if (!data->message.info.mail.message)
		data->message.info.mail.message = gtk_text_buffer_new(NULL);
	text = gtk_text_view_new_with_buffer(data->message.info.mail.message);
	gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(text), GTK_WRAP_WORD_CHAR);
	gtk_container_set_border_width(GTK_CONTAINER(text), 1);

	//gtk_box_pack_start (GTK_BOX (message), text, TRUE, TRUE, 0);
	
	//viewport = gtk_viewport_new(NULL, NULL);
	

	scroll = gtk_scrolled_window_new (NULL, NULL);
	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scroll), GTK_SHADOW_IN);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scroll), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);

	//gtk_container_add (GTK_CONTAINER (viewport), text);	
	gtk_container_add (GTK_CONTAINER (scroll), text);
	

	gtk_box_pack_end (GTK_BOX (page), scroll, TRUE, TRUE, 0);

	data->app_data = app_data;
	data->page = page;
	
	return page;
}
