/* 
 * Folder Signal Emitter for libtinymail
 * Copyright (C) 2007 Milko Krachounov <exabyte@3mhz.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <tny-folder-emitter.h>

static void tny_folder_observer_init (TnyFolderObserverIface *iface);

G_DEFINE_TYPE_WITH_CODE(TnyFolderEmitter, tny_folder_emitter, G_TYPE_OBJECT,
			G_IMPLEMENT_INTERFACE(TNY_TYPE_FOLDER_OBSERVER, tny_folder_observer_init))

enum {
	FOLDER_CHANGED,
	LAST_SIGNAL
};

static guint signals[LAST_SIGNAL] = { 0 };


static void
tny_folder_emitter_update (TnyFolderObserver *self, TnyFolderChange *change)
{
	TNY_FOLDER_EMITTER_GET_CLASS (self)->update_func (self, change);
}

static void
tny_folder_emitter_update_default (TnyFolderObserver *self, TnyFolderChange *change)
{
	g_signal_emit (self, signals[FOLDER_CHANGED], 0, change);
}


static void
tny_folder_emitter_class_init (TnyFolderEmitterClass *class)
{
	signals[FOLDER_CHANGED] =
		g_signal_new ("folder-changed",
			G_OBJECT_CLASS_TYPE (class),
			G_SIGNAL_RUN_LAST,
			G_STRUCT_OFFSET (TnyFolderEmitterClass, folder_changed),
			NULL, NULL,
			g_cclosure_marshal_VOID__OBJECT,
			G_TYPE_NONE, 1, G_TYPE_OBJECT);

	class->update_func = tny_folder_emitter_update_default;

}

static void
tny_folder_emitter_init (TnyFolderEmitter *instance)
{
	/* No initialisation needed */
}


static void
tny_folder_observer_init (TnyFolderObserverIface *iface)
{
	iface->update_func = tny_folder_emitter_update;
}

TnyFolderObserver *
tny_folder_emitter_new ()
{
	return TNY_FOLDER_OBSERVER (g_object_new (TNY_TYPE_FOLDER_EMITTER, NULL));
}


