/* 
 * OpenMoko functions for libtinymail
 * Copyright (C) 2006-2007 Philip Van Hoof <pvanhoof@gnome.org>
 * Copyright (C) 2007-2008 Milko Krachounov <exabyte@3mhz.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
#ifndef TNY_OPENMOKO_ACCOUNT_STORE_H
#define TNY_OPENMOKO_ACCOUNT_STORE_H



#include <glib.h>
#include <glib-object.h>
#include <tny-shared.h>
#include <tny-folder.h>
#include <gconf/gconf-client.h>

#include <tny-camel-shared.h>

G_BEGIN_DECLS

#define TNY_TYPE_OPENMOKO_ACCOUNT_STORE		(tny_openmoko_account_store_get_type ())
#define TNY_OPENMOKO_ACCOUNT_STORE(obj)		(G_TYPE_CHECK_INSTANCE_CAST ((obj), TNY_TYPE_OPENMOKO_ACCOUNT_STORE, TnyOpenMokoAccountStore))
#define TNY_OPENMOKO_ACCOUNT_STORE_CLASS(klass)	(G_TYPE_CHECK_CLASS_CAST ((klass), TNY_TYPE_OPENMOKO_ACCOUNT_STORE, TnyOpenMokoAccountStoreClass))
#define TNY_IS_OPENMOKO_ACCOUNT_STORE(obj)		(G_TYPE_CHECK_INSTANCE_TYPE ((obj), TNY_TYPE_OPENMOKO_ACCOUNT_STORE))
#define TNY_IS_OPENMOKO_ACCOUNT_STORE_CLASS(klass)	(G_TYPE_CHECK_CLASS_TYPE ((klass), TNY_TYPE_OPENMOKO_ACCOUNT_STORE))
#define TNY_OPENMOKO_ACCOUNT_STORE_GET_CLASS(obj)	(G_TYPE_INSTANCE_GET_CLASS ((obj), TNY_TYPE_OPENMOKO_ACCOUNT_STORE, TnyOpenMokoAccountStoreClass))



typedef struct _TnyOpenMokoAccountStore TnyOpenMokoAccountStore;
typedef struct _TnyOpenMokoAccountStoreClass TnyOpenMokoAccountStoreClass;

typedef struct {
	const gchar *id;
	const gchar *name;
	const gchar *address;
	const gchar *replyto;
	const gchar *account_id;
	const gchar *signature;
} Identity;

struct _TnyOpenMokoAccountStore
{
	GObject parent;
};

struct _TnyOpenMokoAccountStoreClass
{
	GObjectClass parent;

	/* signals */
	void	(*account_added)	(TnyOpenMokoAccountStore *self,
					 TnyAccount *account);
	void	(*account_removed)	(TnyOpenMokoAccountStore *self,
					 TnyAccount *account);
	void	(*account_changed)	(TnyOpenMokoAccountStore *self,
					 TnyAccount *account);
	void	(*identity_added)	(TnyOpenMOkoAccountStore *self,
					 Identity *identity);
	void	(*identity_removed)	(TnyOpenMokoAccountStore *self,
					 Identity *identity);
	void	(*identity_changed)	(TnyOpenMokoAccountStore *self,
					 Identity *identity);
};



GType tny_openmoko_account_store_get_type (void);
TnyAccountStore* tny_openmoko_account_store_new (void);
TnySessionCamel* tny_openmoko_account_store_get_session (TnyOpenMokoAccountStore *self);
GConfClient* tny_openmoko_account_store_get_gconf_client (TnyOpenMokoAccountStore *self);


typedef enum {
	TNY_OPENMOKO_ACCOUNT_OBJECT,	/* Don't use, not tested. Doesn't read options either. */
	TNY_OPENMOKO_ACCOUNT_DISABLED, /* Don't use, not implemented. */
	TNY_OPENMOKO_ACCOUNT_TYPE,
	TNY_OPENMOKO_ACCOUNT_PROTO,
	TNY_OPENMOKO_ACCOUNT_MECH,
	TNY_OPENMOKO_ACCOUNT_NAME,
	TNY_OPENMOKO_ACCOUNT_OPTIONS,
	TNY_OPENMOKO_ACCOUNT_USER,
	TNY_OPENMOKO_ACCOUNT_HOSTNAME,
	TNY_OPENMOKO_ACCOUNT_PORT,
	TNY_OPENMOKO_ACCOUNT_URL_STRING,
	TNY_OPENMOKO_ACCOUNT_SENT_FOLDER,
	TNY_OPENMOKO_ACCOUNT_DRAFTS_FOLDER,
	TNY_OPENMOKO_ACCOUNT_TRASH_FOLDER,
	TNY_OPENMOKO_ACCOUNT_IDENTITY,
	TNY_OPENMOKO_ACCOUNT_SKIP_POINTER = -2,
	TNY_OPENMOKO_ACCOUNT_SKIP_INT = -3
} TnyOpenMokoAccountAttribute;



typedef void	(*TnyOpenMokoFindFolderCallback)	(TnyOpenMokoAccountStore *self,
							 gboolean cancelled,
							 TnyFolder *folder,
							 GError *err,
							 gpointer user_data);


typedef void	(*GetIdentititiesFunc)			(TnyOpenMokoAccountStore *self,
							 Identity *identity,
							 gpointer data);


void	tny_openmoko_acount_store_find_folder_async	(TnyOpenMokoAccountStore *self,
							 TnyAccount *account,
							 TnyFolderType folder_type,
							 gboolean create,
							 TnyOpenMokoFindFolderCallback callback,
							 TnyStatusCallback status_callback,
							 gpointer user_data);

const gchar*	tny_openmoko_account_store_get_identity	(TnyOpenMokoAccountStore *self,
						  	TnyAccount *account);

TnyAccount*	tny_openmoko_account_store_get_account_by_id	(TnyOpenMokoAccountStore *self,
								 const gchar *id);

gboolean	tny_openmoko_account_store_add_account_custom	(TnyOpenMokoAccountStore *self,
								 TnyOpenMokoAccountAttribute attr_type,
								 ...);

void		tny_openmoko_account_store_update_account	(TnyOpenMokoAccountStore *self,
								 const gchar *id,
								 TnyOpenMokoAccountAttribute attr_type,
								 ...);

void		tny_openmoko_account_store_get_account_info	(TnyOpenMokoAccountStore *self,
								 const gchar *id,
								 TnyOpenMokoAccountAttribute attr_type,
								 ...);

void		tny_openmoko_account_store_remove_account	(TnyOpenMokoAccountStore *self,
								 const gchar *id);



void		tny_openmoko_account_store_get_identities	(TnyOpenMokoAccountStore *self,
								 GetIdentitiesFunc func,
								 gpointer user_data);

gboolean	tny_openmoko_account_store_add_identity		(TnyOpenMokoAccountStore *self,
								 const gchar *name,
								 const gchar *address,
								 const gchar *replyto,
								 const gchar *account_id,
								 const gchar *signature);


void		tny_openmoko_account_store_remove_identity	(TnyOpenMokoAccountStore *self,
								 const gchar *id);

void		tny_openmoko_account_store_update_identity	(TnyOpenMokoAccountStore *self,
								 const gchar *id,
								 const gchar *name,
								 const gchar *address,
								 const gchar *replyto,
								 const gchar *account_id,
								 const gchar *signature);




G_END_DECLS

#endif
