/* 
 * OpenMoko functions for libtinymail
 * Copyright (C) 2006-2007 Philip Van Hoof <pvanhoof@gnome.org>
 * Copyright (C) 2007 Milko Krachounov <exabyte@3mhz.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#include <tny-openmoko-platform-factory.h>

#include <tny-openmoko-account-store.h>
#include <tny-openmoko-device.h>
#include <tny-camel-mime-part.h>
#include <tny-camel-msg.h>

#include <tny-openmoko-msg-view.h>

#include <tny-gtk-password-dialog.h>

static void tny_platform_factory_init (gpointer g, gpointer iface_data);

G_DEFINE_TYPE_WITH_CODE(TnyOpenMokoPlatformFactory, tny_openmoko_platform_factory, G_TYPE_OBJECT,
			G_IMPLEMENT_INTERFACE(TNY_TYPE_PLATFORM_FACTORY, tny_platform_factory_init))


static void
tny_openmoko_platform_factory_init (TnyOpenMokoPlatformFactory *instance)
{
	return;
}


static TnyMsg*
tny_openmoko_platform_factory_new_msg (TnyPlatformFactory *self)
{
	return tny_camel_msg_new ();
}


static TnyMimePart*
tny_openmoko_platform_factory_new_mime_part (TnyPlatformFactory *self)
{
	return tny_camel_mime_part_new ();
}


static TnyAccountStore*
tny_openmoko_platform_factory_new_account_store (TnyPlatformFactory *self)
{
	return tny_openmoko_account_store_new ();
}

static TnyDevice*
tny_openmoko_platform_factory_new_device (TnyPlatformFactory *self)
{
	return tny_openmoko_device_new ();
}


static TnyPasswordGetter*
tny_openmoko_platform_factory_new_password_getter (TnyPlatformFactory *self)
{
	/* TODO: Implement a better password getter for OpenMoko */
	return tny_gtk_password_dialog_new ();
}

static TnyMsgView*
tny_openmoko_platform_factory_new_msg_view (TnyPlatformFactory *self)
{
	TnyMsgView *retval;
		

	retval = tny_openmoko_msg_view_new ();

	return retval;
}

/**
 * tny_openmoko_platform_factory_get_instance:
 *
 * Get the platform factory singleton instance implemented for OpenMoko based phones
 *
 * Return value: The #TnyPlatformFactory singleton instance implemented for OpenMoko based phones
 **/
TnyPlatformFactory*
tny_openmoko_platform_factory_get_instance (void)
{
	TnyOpenMokoPlatformFactory *self = g_object_new (TNY_TYPE_OPENMOKO_PLATFORM_FACTORY, NULL);

	return TNY_PLATFORM_FACTORY (self);
}


static void
tny_openmoko_platform_factory_finalize (GObject *object)
{
	(G_OBJECT_CLASS(tny_openmoko_platform_factory_parent_class)->finalize) (object);

	return;
}


static void
tny_platform_factory_init (gpointer g, gpointer iface_data)
{
	TnyPlatformFactoryIface *klass = (TnyPlatformFactoryIface *)g;

	klass->new_account_store_func = tny_openmoko_platform_factory_new_account_store;
	klass->new_device_func = tny_openmoko_platform_factory_new_device;
	klass->new_msg_view_func = tny_openmoko_platform_factory_new_msg_view;
	klass->new_msg_func = tny_openmoko_platform_factory_new_msg;
	klass->new_mime_part_func = tny_openmoko_platform_factory_new_mime_part;
	klass->new_password_getter_func = tny_openmoko_platform_factory_new_password_getter;

	return;
}


static TnyOpenMokoPlatformFactory *the_singleton = NULL;


static GObject*
tny_openmoko_platform_factory_constructor (GType type, guint n_construct_params,
			GObjectConstructParam *construct_params)
{
	GObject *object;

	/* TODO: potential problem: singleton without lock */

	if (G_UNLIKELY (!the_singleton))
	{
		object = G_OBJECT_CLASS (tny_openmoko_platform_factory_parent_class)->constructor (type, n_construct_params, construct_params);

		the_singleton = TNY_OPENMOKO_PLATFORM_FACTORY (object);
	}
	else
	{
		/* refdbg killed bug! 
		object = g_object_ref (G_OBJECT (the_singleton)); */

		object = G_OBJECT (the_singleton);
		g_object_freeze_notify (G_OBJECT(the_singleton));
	}

	return object;
}

static void 
tny_openmoko_platform_factory_class_init (TnyOpenMokoPlatformFactoryClass *class)
{
	GObjectClass *object_class;
	object_class = (GObjectClass*) class;

	object_class->finalize = tny_openmoko_platform_factory_finalize;
	object_class->constructor = tny_openmoko_platform_factory_constructor;

	return;
}


