/*
 * Messages for OpenMoko
 * Copyright (C) 2007 Milko Krachounov <exabyte@3mhz.net>
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */


#include <tny-openmoko-platform-factory.h>
#include <tny-camel-transport-account.h>
#include <tny-camel-send-queue.h>
#include <tny-gtk-account-list-model.h>
#include <tny-openmoko-account-store.h>
#include <string.h>


#include "mail-defines.h"
#include "mail-common.h"

TnySendQueue*
get_send_queue(TnyTransportAccount *account, AppData *data)
{
	GList *walk;
	TnyCamelTransportAccount *account_walk;
	//const gchar* id = tny_account_get_id(TNY_ACCOUNT(account));

	for (walk = data->queues; walk; walk = g_list_next(walk)) {
		account_walk = tny_camel_send_queue_get_transport_account(TNY_CAMEL_SEND_QUEUE(walk->data));

		//if (!strcmp(tny_account_get_id(TNY_ACCOUNT(account_walk)), id))
		if (TNY_ACCOUNT(account) == TNY_ACCOUNT(account_walk))
		{
			g_object_unref(G_OBJECT(account_walk));
			return TNY_SEND_QUEUE(g_object_ref(G_OBJECT(walk->data)));
		}	
		g_object_unref(G_OBJECT(account_walk));	
	}
	return NULL;
}


static void mail_init_send_queues (AppData *data)
{
	TnyTransportAccount *account;
	TnySendQueue *queue;
	TnyIterator *iter;

	data->queues = NULL;
	data->transport_accounts = tny_gtk_account_list_model_new();

	if (data->account_store)
		tny_account_store_get_accounts (data->account_store, TNY_LIST(data->transport_accounts), TNY_ACCOUNT_STORE_TRANSPORT_ACCOUNTS);

	iter = tny_list_create_iterator (TNY_LIST(data->transport_accounts));

	while (!tny_iterator_is_done (iter)) {
		account = TNY_TRANSPORT_ACCOUNT (tny_iterator_get_current (iter));
		queue = tny_camel_send_queue_new(TNY_CAMEL_TRANSPORT_ACCOUNT(account));
		data->queues = g_list_prepend(data->queues, queue);

		g_object_unref (G_OBJECT (account));
		tny_iterator_next (iter);
	}

	g_object_unref (G_OBJECT (iter));

	

}

/*

	gconftool-2 -s /apps/tinymail/identities/0/name -t string "My Name"
	gconftool-2 -s /apps/tinymail/identities/0/address -t string me@example.com
	#gconftool-2 -s /apps/tinymail/identities/0/replyto -t string replyto@example.com
	gconftool-2 -s /apps/tinymail/identities/0/account_id -t string 10
	#gconftool-2 -s /apps/tinymail/identities/0/replyto -t string replyto@example.com
	#gconftool-2 -s /apps/tinymail/identities/0/signature -t string "I'm dead"

*/

static void
add_identity (TnyOpenMokoAccountStore *self, Identity *identity, GtkListStore *identities_list)
{
	TnyTransportAccount *account;
	if (identity->account_id && identity->account_id[0])
		account = TNY_TRANSPORT_ACCOUNT(tny_openmoko_account_store_get_account_by_id(self, identity->account_id));

	if (identity->address && identity->address[0] && identity->name && identity->name[0]) {
		identity = g_strdup_printf("%s <%s>", identity->name, identity->address);
		gtk_list_store_insert_with_values (identities_list,
						   NULL, -1,
						   IDENTITIES_COLUMN_ID, identity->id,
						   IDENTITIES_COLUMN_NAME, identity,
						   IDENTITIES_COLUMN_REPLYTO, identity->replyto,
						   IDENTITIES_COLUMN_SIGNATURE, identity->signature,
						   IDENTITIES_COLUMN_ACCOUNT, account,
						   -1);
		g_free(identity);
	}
}
gboolean
remove_identity_from_list (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, const gchar *id)
{
	gboolean result = FALSE;
	gchar *this_row_id;
	gtk_tree_model_get (model, iter, IDENTITIES_COLUMN_ID, &this_row_id, -1);
	if (!strcmp(id, this_row_id)) {
		result = gtk_list_store_remove(GTK_LIST_STORE(model), iter);
	}
	g_free(this_row_id);
	return result;
}

static void
remove_identity (TnyOpenMokoAccountStore *self, Identity *identity, GtkListStore *identities_list)
{
	gtk_tree_model_foreach (GTK_TREE_MODEL(identities_list),
	                        (GtkTreeModelForeachFunc*) remove_identity_from_list,
                                identity->id);
}

static void
mail_init_identities (AppData *data)
{
	GtkListStore *identities_list;

	TnyIterator *iter;


	identities_list = gtk_list_store_new(IDENTITIES_N_COLUMNS,
					G_TYPE_STRING,
					G_TYPE_STRING,
					G_TYPE_STRING,
					G_TYPE_STRING,
					G_TYPE_OBJECT);

	data->identities = GTK_TREE_MODEL(identities_list);

	if (!data->account_store)
		return;

	/* TODO */
	data->default_identity = gconf_client_get_string (client, "/apps/tinymail/default_identity", NULL);
	if (data->default_identity && !*data->default_identity) {
		g_free(data->default_identity);
		data->default_identity = NULL;
	}

	tny_openmoko_account_store_get_identities(TNY_OPENMOKO_ACCOUNT_STORE(data->account_store),
	                                          (GetIdentitiesFunc*) add_identity,
	                                          data->identities);
	g_signal_connect(G_OBJECT(data->account_store), "identity-added",
	                 G_CALLBACK(add_identity), data->folder_list);
	g_signal_connect(G_OBJECT(data->account_store), "identity-removed",
	                 G_CALLBACK(remove_identity), data->folder_list);

}

void messages_access_init(AppData *data)
{
	TnyPlatformFactory* platfact;

	platfact = tny_openmoko_platform_factory_get_instance();
	data->account_store = tny_platform_factory_new_account_store(platfact);
	data->platfact = platfact;

	mail_init_send_queues(data);
	mail_init_identities(data);
}
