/*
 * Mail for OpenMoko
 * Copyright (C) 2007 Milko Krachounov <exabyte@3mhz.net>
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include <gtk/gtk.h>
#include <strings.h>
#include <glib/gi18n.h>
#include <moko-hint-entry.h>

#include <tny-folder.h>
#include <tny-gtk-folder-store-tree-model.h>

#include "mail-defines.h"
#include "mail-common.h"
#include "mail-access.h"
#include "menutoolbutton.h"
#include "mokofieldlabel.h"

gchar *omail_ui_image_path;

void init_app_data(AppData *app_data)
{
	/* We need no actual initialisation, beside zeroing. */
	bzero(app_data, sizeof(AppData));
}

GdkPixbuf *
pixbuf_load_from_datadir(const gchar *name, GError **error)
{
	GdkPixbuf *pixbuf;
	gchararray image_file_name;

	image_file_name = g_build_path("/", omail_ui_image_path, name, NULL);
	pixbuf = gdk_pixbuf_new_from_file(image_file_name, error);
	g_free(image_file_name);
	return pixbuf;
}


gint
notebook_add_page_with_icon (GtkNotebook *notebook, GtkWidget *child, const gchar *icon_name, int padding)
{
	gint page_num;
	GtkWidget *icon = gtk_image_new_from_icon_name (icon_name, GTK_ICON_SIZE_LARGE_TOOLBAR);
	GtkWidget *align = gtk_alignment_new (0.5, 0.5, 1.0, 1.0);
	gtk_alignment_set_padding (GTK_ALIGNMENT (align), padding, padding, padding, padding);
	gtk_container_add (GTK_CONTAINER (align), icon);
	gtk_widget_show_all (align);
	page_num = gtk_notebook_append_page (notebook, child, align);
	gtk_container_child_set (GTK_CONTAINER (notebook), child, "tab-expand", TRUE, NULL);
	return page_num;
}


void 
toolbar_insert_menu_button (GtkToolbar* toolbar, GtkWidget *icon, GCallback callback, gpointer data, GtkWidget* menu)
{
	
	GtkToolItem *button = moko_menu_tool_button_new(icon, "");
	moko_menu_tool_button_set_menu  (MOKO_MENU_TOOL_BUTTON(button), menu);
	g_signal_connect (G_OBJECT (button), "clicked", callback, data);
	gtk_tool_item_set_expand (button, TRUE);
	if (gtk_toolbar_get_n_items(GTK_TOOLBAR(toolbar)))
		gtk_toolbar_insert (GTK_TOOLBAR (toolbar), gtk_separator_tool_item_new (), -1);

	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), button, -1);	
}


void 
toolbar_insert_button (GtkToolbar* toolbar, GtkWidget *icon, GCallback callback, gpointer data)
{
	
	GtkToolItem *button = gtk_tool_button_new(icon, "");
	g_signal_connect (G_OBJECT (button), "clicked", callback, data);
	gtk_tool_item_set_expand (button, TRUE);
	if (gtk_toolbar_get_n_items(GTK_TOOLBAR(toolbar)))
		gtk_toolbar_insert (GTK_TOOLBAR (toolbar), gtk_separator_tool_item_new (), -1);

	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), button, -1);	
}

void 
toolbar_insert_toggle_button (GtkToolbar* toolbar, GtkWidget *icon, GCallback callback, gpointer data)
{
	
	GtkToolItem *button = gtk_toggle_tool_button_new();
	gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(button), icon);
	g_signal_connect (G_OBJECT (button), "toggled", callback, data);
	gtk_tool_item_set_expand (button, TRUE);
	if (gtk_toolbar_get_n_items(GTK_TOOLBAR(toolbar)))
		gtk_toolbar_insert (GTK_TOOLBAR (toolbar), gtk_separator_tool_item_new (), -1);

	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), button, -1);	
}

#if 0
void
toolbar_insert_button_from_stock (GtkWidget *toolbar, const gchar *icon_name, GCallback callback, gpointer data)
{
	GtkToolItem *button = gtk_tool_button_new_from_stock (icon_name);
	g_signal_connect (G_OBJECT (button), "clicked", callback, data);
	
	gtk_tool_item_set_expand (button, TRUE);
	if (gtk_toolbar_get_n_items(GTK_TOOLBAR(toolbar)))
		gtk_toolbar_insert (GTK_TOOLBAR (toolbar), gtk_separator_tool_item_new (), 0);

	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), button, 0);

}
#endif


/* We need a function to remove all widgets from a container */
/* Callback used for mass removal of widgets from a container */
static void
cb_widget_remove (GtkWidget* widget, GtkContainer *container)
{
	gtk_container_remove(container, widget);
}

void
container_remove_all_widgets (GtkContainer *container)
{
	gtk_container_foreach (container, (GtkCallback)cb_widget_remove, container);
}


/* TODO: Move to access */
gchararray
get_contact_name (gchararray number)
{
	/* TODO: Implement me */
	return number;
}


/* Choose a folder */
gint
folder_select_dialog (const gchar* text, TnyFolder **result, AppData *app_data)
{
	TnyFolder *folder = NULL;
	GtkWidget *dialog, *widget, *w;
	GtkCellRenderer *renderer;
	gint dialog_result, r;

	GtkTreeIter iter;
	gint element_type;

	
	dialog = gtk_dialog_new_with_buttons(_("Select folder"),
	                                     NULL,
	                                     GTK_DIALOG_DESTROY_WITH_PARENT,
	                                     GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
	                                     GTK_STOCK_OK, GTK_RESPONSE_OK, NULL);

	gtk_dialog_set_alternative_button_order(GTK_DIALOG(dialog), GTK_RESPONSE_OK, GTK_RESPONSE_CANCEL);

	widget = moko_field_label_new(text);
	gtk_widget_show(widget);
	gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox), widget);

	widget = gtk_combo_box_new_with_model (app_data->folder_list);
	gtk_widget_show(widget);
	gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox), widget);


	renderer = gtk_cell_renderer_text_new();
	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (widget), renderer, TRUE);
	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (widget),
	                                renderer,
	                                "text", TNY_GTK_FOLDER_STORE_TREE_MODEL_NAME_COLUMN,
	                                NULL);

	/* TODO */
	gtk_combo_box_set_active(GTK_COMBO_BOX(widget), 0);

	dialog_result = gtk_dialog_run(GTK_DIALOG(dialog));



	if (gtk_combo_box_get_active_iter(GTK_COMBO_BOX(widget), &iter)) {
		gtk_tree_model_get(app_data->folder_list,
				   &iter,
				   TNY_GTK_FOLDER_STORE_TREE_MODEL_TYPE_COLUMN,
				   &element_type,
				   TNY_GTK_FOLDER_STORE_TREE_MODEL_INSTANCE_COLUMN,
				   &folder,
				   -1);

		if (element_type == TNY_FOLDER_TYPE_ROOT) {
			if (dialog_result == GTK_RESPONSE_OK) {
				quick_dialog_run(w, r, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
					 	_("The selected element is not a folder, but an account"));
			}
			g_object_unref(folder);
			folder = NULL;
		}

	}

	gtk_widget_destroy(dialog);
	
	if (result)
		*result = folder;

	return dialog_result;

}



/* Ask the user for something */
gint
text_entry_dialog (const gchar* text, gchar **result, const gchar *default_text)
{
	GtkWidget *dialog, *widget;
	gint dialog_result;


	
	dialog = gtk_dialog_new_with_buttons(_("Text input"),
	                                     NULL,
	                                     GTK_DIALOG_DESTROY_WITH_PARENT,
	                                     GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
	                                     GTK_STOCK_OK, GTK_RESPONSE_OK, NULL);

	gtk_dialog_set_alternative_button_order(GTK_DIALOG(dialog), GTK_RESPONSE_OK, GTK_RESPONSE_CANCEL);

	widget = moko_field_label_new(text);
	gtk_widget_show(widget);
	gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox), widget);

	widget = gtk_entry_new();
	if (default_text)
		gtk_entry_set_text(GTK_ENTRY(widget), default_text);
	gtk_widget_show(widget);
	gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox), widget);

	dialog_result = gtk_dialog_run(GTK_DIALOG(dialog));

	if (result)
		*result = g_strdup(gtk_entry_get_text(GTK_ENTRY(widget)));

	gtk_widget_destroy(dialog);
	
	return dialog_result;

}

void
status_update (GObject *sender, TnyStatus *status, AppData *data)
{
}

typedef struct {
	const gchar* name;
	off_t size;
} SizeUnit;


/* We need to convert file sizes to human readable format */
/* This should be in order, and the last should be NULL, -1 */
static SizeUnit binary_units[] = {	
	{ N_("B"), 1 },
	{ N_("KB"), 1024 },
	{ N_("MB"), 1048576 },
	{ N_("GB"), 1073741824 },
	{ NULL, -1 }
};

/* 
 *  This function returns the suitable factor, which we should divide the size
 *  by and sets *unit to the respective units used
 */

off_t
get_size_display_factor(off_t size, const gchar **unit)
{
	gint i;
	off_t factor;
	const gchar* name;
	i = 0;
	do {
		name = binary_units[i].name;
		factor = binary_units[i].size;
	} while (size / binary_units[++i].size >= 1);

	if (unit)
		*unit = name;
	return factor;
}

/* A function, which converts the date to a human-readable format,
   do not free, modify or store the result */
const gchar*
date_human_readable(time_t ts)
{
	/* Do not reduce this, it is possible that the date is more bytes
           in some locales, while I highly doubt it, 128 is not that much. */
	static gchar buf[128];
	struct tm tmp;
	localtime_r(&ts, &tmp);
	strftime(buf, sizeof(buf), "%c", &tmp);
	return buf;
}

gboolean
is_address_field_valid (const gchar *field, gboolean single)
{
	/* TODO: Implement */
	return field && *field;

}


