/*
 * Mail for OpenMoko
 * Copyright (C) 2007 Milko Krachounov <exabyte@3mhz.net>
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */


#ifndef MAIL_COMMON_H
#define MAIL_COMMON_H

#include <gtk/gtk.h>
#include <sys/types.h>
#include <tny-folder.h>
#include <tny-header.h>
#include <tny-msg.h>
#include <tny-send-queue.h>
#include <tny-platform-factory.h>
#include <tny-gtk-header-list-model.h>

extern gchar *omail_ui_image_path;

/* Struct for application-wide data */

/* Currently half of this are not ref'd. This is not a problem with the current design, but
   could lead to problems in the future. */
typedef struct {
	
	GtkWindow *window;

	GtkNotebook *notebook;

	GtkTreeModel *folder_list;

	GtkTreeModel *identities;
	GtkTreeModel *transport_accounts;
	GList *queues;

	/* The following is the default identity, if it exists it is always chosen when sending a message */
	gchar* default_identity;	

	TnyPlatformFactory* platfact;
	TnyAccountStore *account_store;

	/* Hash table of folder observers, that are used for performing operations on system folders,
	   like moving the messages added to the internal sent folder and adding the appropriate flags
	   to the messages added to the drafts folder. The data in the hash is contained in an struct
	   containing an unreferenced observer (weak reference is used to remove it from the hash)
	   and some data. */
	GHashTable *observers;

	/* These are only internally used in mail-list.c */
	/* Currently selected element */
	TnyFolder *current_folder;
	TnyFolderType folder_type;
	TnyHeader *current_header;

	/* Observer */
	TnyFolderObserver *folder_monitor;

	/* List renderer, so we can change the settings for different folders */
	GtkCellRenderer *renderer;

	/* Our message list models */
	GtkTreeModel *message_filter;

	/* Stuff for our filter */
	gchararray message_filter_text;
	gboolean message_filter_active;
	
	GtkTreeSelection *message_selection;

	GtkComboBox *folder_combo;
	GtkTreeView *message_view;

	/* This are for multiple message selection, widget with helper buttons ("virtual keyboard") and 
	   the modifiers pressed on it */
	guint modifiers;
	GtkWidget *selector;	
	GtkWidget *shift_toggle;


	
} AppData;


enum {
	IDENTITIES_COLUMN_ID = 0,
	IDENTITIES_COLUMN_NAME,
	IDENTITIES_COLUMN_REPLYTO,
	IDENTITIES_COLUMN_SIGNATURE,
	IDENTITIES_COLUMN_ACCOUNT,
	
	IDENTITIES_N_COLUMNS
};

typedef struct {
	TnyFolderObserver *observer;
	gpointer data;
} FolderObserverData;


void init_app_data(AppData *app_data);

GdkPixbuf *pixbuf_load_from_datadir(const gchar *name, GError **error);

gint
notebook_add_page_with_icon (GtkNotebook *notebook,
			     GtkWidget *child,
			     const gchar *icon,
			     gint padding);
void
toolbar_insert_button (GtkToolbar*, GtkWidget *icon, GCallback, gpointer data);
void
toolbar_insert_menu_button (GtkToolbar*, GtkWidget *icon, GCallback, gpointer data, GtkWidget* menu); 
void 
toolbar_insert_toggle_button (GtkToolbar* toolbar, GtkWidget *icon, GCallback callback, gpointer data);

void container_remove_all_widgets (GtkContainer *container);


gchararray	get_contact_name (gchararray number);


/* This functions query the user for folder or text, which is saved in *result (if result != NULL)
   They return either GTK_RESPONSE_OK or GTK_RESPONSE_CANCEL. The result should be unref'd or freed,
   if it is not NULL, even if GTK_RESPONSE_CANCEL is received. */
gint		folder_select_dialog (const gchar* text, TnyFolder **result, AppData *data);
gint		text_entry_dialog (const gchar* text, gchar **result, const gchar *default_text);

off_t		get_size_display_factor(off_t size, const gchar **unit);
/* A function, which converts the date to a human-readable format,
   do not free, modify or store the result */
const gchar* date_human_readable(time_t ts);

void status_update (GObject *sender, TnyStatus *status, AppData *data);

/* Checks whether the address field is valid, single should be TRUE if the field accepts only one address */

gboolean is_address_field_valid (const gchar *field, gboolean single);


/* NOTE: Be careful when using these macros. They do not include { and } */
#define quick_dialog_run(dialog, result, type, buttons, ...) \
	dialog = gtk_message_dialog_new(NULL, \
					GTK_DIALOG_DESTROY_WITH_PARENT, \
					type, buttons, \
					__VA_ARGS__); \
	result = gtk_dialog_run(GTK_DIALOG(dialog)); \
	gtk_widget_destroy(dialog)

#define invalid_field_dialog(dialog, dialog_result, field) \
	quick_dialog_run(dialog, dialog_result, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, \
			 _("The %s field doesn't contain a valid email address"), field)


#endif


