/*
 * Mail for OpenMoko
 * Copyright (C) 2008 Milko Krachounov <exabyte@3mhz.net>
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include <gtk/gtk.h>
#include <glib/gi18n.h>
#include <moko-finger-scroll.h>
#include <moko-stock.h>
#include <tny-account.h>
#include <tny-store-account.h>
#include <tny-openmoko-account-store.h>
#include <tny-gtk-account-list-model.h>

#include "mail-edit-account.h"

typedef struct {
	AppData *app_data;
	GtkTreeSelection *selection;
	GtkWidget *page;
} ConfigPageData;


static void
cb_close_page (GtkToolButton *button, ConfigPageData *data)
{
	g_object_unref(data->selection);
	gtk_widget_destroy(data->page);
	g_free(data);
}

static void
cb_new_account (GtkToolButton *button, ConfigPageData *data)
{
	gint page_num;
	GtkWidget *page;
	page = messages_create_edit_account_page(data->app_data, NULL, MAIL_ACCOUNT_EDIT_INCOMING_PAGE);
	page_num = notebook_add_page_with_icon (data->app_data->notebook, page, GTK_STOCK_EDIT, 6);
	gtk_notebook_set_current_page (data->app_data->notebook, page_num);
}

static void
cb_edit_account (GtkToolButton *button, ConfigPageData *data)
{
	GtkTreeIter iter;
	GtkTreeModel *model;
	
	if (gtk_tree_selection_get_selected (data->selection, &model, &iter)) {
		gint page_num;
		GtkWidget *page;
		TnyAccount *account;

		gtk_tree_model_get (model, &iter, TNY_GTK_ACCOUNT_LIST_MODEL_INSTANCE_COLUMN, &account, -1);

		page = messages_create_edit_account_page(data->app_data, account, MAIL_ACCOUNT_EDIT_INCOMING_PAGE);
		page_num = notebook_add_page_with_icon (data->app_data->notebook, page, GTK_STOCK_EDIT, 6);
		gtk_notebook_set_current_page (data->app_data->notebook, page_num);
		
		g_object_unref(account);
	}
}

static void
cb_delete_account (GtkToolButton *button, ConfigPageData *data)
{
	GtkTreeIter iter;
	GtkTreeModel *model;
	
	if (gtk_tree_selection_get_selected (data->selection, &model, &iter)) {

		TnyAccount *account;

		gtk_tree_model_get (model, &iter, TNY_GTK_ACCOUNT_LIST_MODEL_INSTANCE_COLUMN, &account, -1);

		tny_openmoko_account_store_remove_account (TNY_OPENMOKO_ACCOUNT_STORE(data->app_data->account_store), tny_account_get_id(account));
		
		g_object_unref(account);
	}
}

static void
add_account_to_model (TnyAccountStore *store, TnyAccount *account, GtkTreeModel *account_list)
{
	if (TNY_IS_STORE_ACCOUNT(account))
		tny_list_append(TNY_LIST(account_list), G_OBJECT(account_list));
}

static void
remove_account_from_model (TnyAccountStore *store, TnyAccount *account, GtkTreeModel *account_list)
{
	tny_list_remove(TNY_LIST(account_list), G_OBJECT(account));
}

/* Due to simplicity, I display only the store accounts and have a transport account
   tied to each store one. */
GtkWidget*
messages_create_config_page (AppData* app_data)
{
	GtkWidget *page, *toolbar, *scroll, *account_view;
	GtkTreeModel *accounts;
	GtkCellRenderer *renderer;

	ConfigPageData *data = g_malloc0(sizeof(ConfigPageData));
	data->app_data = app_data;

	page = gtk_vbox_new (FALSE, 0);

	toolbar = gtk_toolbar_new ();

	toolbar_insert_button (GTK_TOOLBAR(toolbar), gtk_image_new_from_icon_name (GTK_STOCK_NEW, GTK_ICON_SIZE_LARGE_TOOLBAR), G_CALLBACK(cb_new_account), data);
	toolbar_insert_button (GTK_TOOLBAR(toolbar), gtk_image_new_from_icon_name (GTK_STOCK_EDIT, GTK_ICON_SIZE_LARGE_TOOLBAR), G_CALLBACK(cb_edit_account), data);
	toolbar_insert_button (GTK_TOOLBAR(toolbar), gtk_image_new_from_icon_name (GTK_STOCK_DELETE, GTK_ICON_SIZE_LARGE_TOOLBAR), G_CALLBACK(cb_delete_account), data);
	toolbar_insert_button (GTK_TOOLBAR(toolbar), gtk_image_new_from_icon_name (GTK_STOCK_CLOSE, GTK_ICON_SIZE_LARGE_TOOLBAR), G_CALLBACK(cb_close_page), data);

	gtk_widget_show_all (toolbar);

	gtk_box_pack_start (GTK_BOX (page), toolbar, FALSE, TRUE, 0);

	scroll = moko_finger_scroll_new();
	account_view = gtk_tree_view_new ();
	
	/* FIXME: The following is an ugly hack, which improves performance when using 
           the finger scroll by killing prelight redrawing of the entries. */
	g_signal_connect (G_OBJECT(account_view), "enter-notify-event", G_CALLBACK (gtk_true), NULL);
	g_signal_connect (G_OBJECT(account_view), "motion-notify-event", G_CALLBACK (gtk_true), NULL);

	gtk_container_add (GTK_CONTAINER (scroll), account_view);

	gtk_widget_show_all(scroll);

	gtk_box_pack_start(GTK_BOX(page), scroll, TRUE, TRUE, 0);

	/* We initialise our account list :) */
	/* TODO: I want an account filter, which enables switching the display
	   of transport accounts (advanced mode) and adds an icon to the accounts. 
	   DON'T FORGET TO UPDATE THE CALLBACK, TOO.
	*/
	accounts = tny_gtk_account_list_model_new ();
	tny_account_store_get_accounts (app_data->account_store, TNY_LIST (accounts), TNY_ACCOUNT_STORE_STORE_ACCOUNTS);
	
	g_signal_connect(G_OBJECT(app_data->account_store), "account-added",
	                 G_CALLBACK(add_account_to_model), accounts);
	g_signal_connect(G_OBJECT(app_data->account_store), "account-removed",
	                 G_CALLBACK(remove_account_from_model), accounts);
	
	renderer = gtk_cell_renderer_text_new();
	gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW (account_view),
	                                            -1,
	                                            _("Account"),
	                                            renderer,
	                                            "text", TNY_GTK_ACCOUNT_LIST_MODEL_NAME_COLUMN,
	                                            NULL);


	gtk_tree_view_set_model (GTK_TREE_VIEW (account_view), accounts);
	
	g_object_unref(accounts);

	data->selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (account_view));
	
	gtk_widget_show (page);

	return page;

}

