/*
 * Mail for OpenMoko
 * Copyright (C) 2008 Milko Krachounov <exabyte@3mhz.net>
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include <gtk/gtk.h>
#include <glib/gi18n.h>
#include <moko-finger-scroll.h>
#include <moko-stock.h>
#include <tny-account.h>
#include <tny-store-account.h>
#include <tny-openmoko-account-store.h>
#include <tny-gtk-account-list-model.h>

#include "mail-edit-account.h"

typedef struct {
	AppData *app_data;
	TnyAccount *store_account;
	TnyAccount *transport_account;
#if 0
	TNY_OPENMOKO_ACCOUNT_OBJECT,	/* Don't use, not tested. Doesn't read options either. */
	TNY_OPENMOKO_ACCOUNT_DISABLED, /* Don't use, not implemented. */
	TNY_OPENMOKO_ACCOUNT_TYPE,
	TNY_OPENMOKO_ACCOUNT_PROTO,
	TNY_OPENMOKO_ACCOUNT_MECH,
	TNY_OPENMOKO_ACCOUNT_NAME,
	TNY_OPENMOKO_ACCOUNT_OPTIONS,
	TNY_OPENMOKO_ACCOUNT_USER,
	TNY_OPENMOKO_ACCOUNT_HOSTNAME,
	TNY_OPENMOKO_ACCOUNT_PORT,
	TNY_OPENMOKO_ACCOUNT_URL_STRING,
	TNY_OPENMOKO_ACCOUNT_SENT_FOLDER,
	TNY_OPENMOKO_ACCOUNT_DRAFTS_FOLDER,
	TNY_OPENMOKO_ACCOUNT_TRASH_FOLDER,
	TNY_OPENMOKO_ACCOUNT_IDENTITY
#endif
	GtkWidget *name;
	GtkWidget *in_proto;
	GtkWidget *in_hostname;
	GtkWidget *in_port;
	GtkWidget *in_user;
	GtkWidget *in_ssl;
	GtkWidget *out_hostname;
	GtkWidget *out_port;
	/*GtkWidget *out_auth;*/
	GtkWidget *out_user;
	GtkWidget *out_ssl;

	GtkWidget *sent_folder; gboolean sent_folder_altered;
	GtkWidget *drafts_folder; gboolean drafts_folder_altered;
	GtkWidget *trash_folder; gboolean trash_folder_altered;

	GSList *identities;
	gint n_identities;
	/* Pack identities here */
	GtkBox *identity_box;	
	
	GtkWidget *page;
} EditAccountPageData;

enum {
	ACCOUNT_INFO_VALID = 0,
	STORE_ACCOUNT_INVALID = 1<<0,
	STORE_ACCOUNT_EMPTY = 1<<1,
	TRANSPORT_ACCOUNT_INVALID = 1<<2,
	TRANSPORT_ACCOUNT_EMPTRY = 1<<3
};

typedef struct {
	const gchar *name;
	const gchar *value;
} ValueMap;

static ValueMap protos[] = {
	{ "POP3", "pop" },
	{ "IMAP", "imap" },
	{ NULL, NULL }
};

static ValueMap encryption[] = {
	{ "No encryption", "never" },	
	{ "Require TLS", "tls" },
	{ "SSL", "wrapped" },
	{ "TLS when possible", "when-possible" },
	{ NULL, NULL }
};


static void
cb_status_edit_account (GObject *sender, TnyStatus *status, EditAccountPageData *data)
{
	status_update(sender, status, data->app_data);
}

static void
set_to_true (gboolean *var)
{
	*var = TRUE;
}

static void
combo_fill_in (GtkComboBox *combo, ValueMap *map) {
	while (map->name) {
		gtk_combo_box_append_text (combo, map->name);
		map++;
	}
}

static const gchar*
find_combo_value (GtkComboBox *combo, ValueMap *map) {
	gchar *text;
	const gchar *result = NULL;

	text = gtk_combo_box_get_active_text (combo);

	while (map->name) {
		if (!strcmp(map->name, text))
			result = map->value;
		map++;
	}
	g_free(text);

	return result;
}

static gboolean
set_combo_value (GtkComboBox *combo, ValueMap *map, const gchar *value) {
	int pos = 0;
	while (map->name) {
		if (!strcmp(map->value, value))
			break;
		pos++;
		map++;
	}
	gtk_combo_box_set_active (combo, pos);

}

static void
unset_combo_box(GtkComboBox *box)
{
	gtk_combo_box_set_active (box, -1);
}



/* GtkTreeModelForeachFunc to set the folder */
typedef struct {
	TnyFolder *folder;
	GtkComboBox *combo;
} SetFolderData;

static gboolean
set_folder_combo_to (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, SetFolderData data)
{
	TnyFolder *cur_folder;
	gtk_tree_model_get(app_data->folder_list,
			   &iter,
			   TNY_GTK_FOLDER_STORE_TREE_MODEL_INSTANCE_COLUMN,
			   &cur_folder,
			   -1);

	/* We'll just compare the pointer, so we don't need this. */
	g_object_unref(cur_folder);
	if (cur_folder == data->folder) {
		gtk_combo_box_set_active_iter(data->combo, iter);
		return TRUE;
	}
	return FALSE;
}



static void
add_identity(EditAccountPageData *data, const gchar *identity)
{
	gchar hint;
	GtkWidget *entry;

	hint = g_strdup_printf(_("Identity %d"), ++data->n_identities);
	entry = moko_hint_entry_new(hint);
	g_free(hint);

	if (identity)
		gtk_entry_set(GTK_ENTRY(entry), identity);

	data->identities = g_slist_append(data->identitites, GTK_ENTRY(entry));
	gtk_box_pack_start(data->identity_box, entry, FALSE, TRUE, 0);
}

static void
find_transport_account(EditAccountPageData *data)
{
	/* We search for the identity of the account and then get its transport. Lame. */
	const gchar* identity = tny_openmoko_account_store_get_identity(TNY_OPENMOKO_ACCOUNT_STORE(data->app_data->account_store), data->store_account);
	
	GtkTreeIter iter;
	/* We try to find our identity the lame way */
	if (gtk_tree_model_get_iter_first (data->app_data->identities, &iter)) do
	{
		gchar *walk_identity;
		TnyAccount *account;

		gtk_tree_model_get(data->app_data->identities,
				   &iter,
				   IDENTITIES_COLUMN_ID,
				   &walk_identity,
				   IDENTITIES_COLUMN_ACCOUNT,
				   &account,
				   -1);

		if (walk_identity && !strcmp(identity, walk_identity)) {
			g_free(walk_identity);
			data->transport_account = account;
			break;
		}
		if (walk_account)
			g_object_unref(walk_account);
		g_free(walk_identity);
	} while (gtk_tree_model_iter_next (data->app_data->identities, &iter));
}

static void
find_identities(EditAccountPageData *data)
{
	GtkTreeIter iter;
	/* We search for all identities that use this outgoing account */
	if (gtk_tree_model_get_iter_first (data->app_data->identities, &iter)) do
	{
		gchar *identity;
		TnyAccount *account;

		gtk_tree_model_get(data->app_data->identities,
				   &iter,
				   IDENTITIES_COLUMN_ID,
				   &identity,
				   IDENTITIES_COLUMN_ACCOUNT,
				   &account,
				   -1);
		
		/* We'll only compare pointer here, so not needed... */
		if (walk_account)
			g_object_unref(walk_account);

		if (!identity)
			continue;
		
		if (account == data->transport_account)
			add_identity(identity);

		g_free(identity);
	} while (gtk_tree_model_iter_next (data->app_data->identities, &iter));
}


static void
load_store_account_data(EditAccountPageData *data)
{
	if (data->store_account) {

		gchararray name, proto, hostname, user, port_string;
		gint port;
		GSList *options, *walk;


		tny_openmoko_account_store_get_account_info	(TNY_OPENMOKO_ACCOUNT_STORE(data->app_data->account_store),
								 tny_account_get_id(data->store_account),
								 TNY_OPENMOKO_ACCOUNT_NAME, &name,
								 TNY_OPENMOKO_ACCOUNT_PROTO, &proto,
								 TNY_OPENMOKO_ACCOUNT_HOSTNAME, &hostname,
								 TNY_OPENMOKO_ACCOUNT_PORT, &port,
								 TNY_OPENMOKO_ACCOUNT_USER, &user,
								 TNY_OPENMOKO_ACCOUNT_OPTIONS, &options,
								 -1);

		port_string = g_strdup_printf("%d", port);

		gtk_entry_set_text(GTK_ENTRY(data->name), name);
		set_combo_value(GTK_COMBO_BOX(data->in_proto), protos, proto);
		gtk_entry_set_text(GTK_ENTRY(data->in_hostname), hostname);
		gtk_entry_set_text(GTK_ENTRY(data->in_port), port_string);
		gtk_entry_set_text(GTK_ENTRY(data->in_user), user);
		for (walk = options; walk; walk = options->next) {
			if (!strncmp(walk->data, "use_ssl=", 8))
				set_combo_value(GTK_COMBO_BOX(data->in_ssl), encryption, strchr(walk->data, '=') + 1);
			g_free(walk->data);
		}
		g_slist_free(options);
		g_free(name);
		g_free(proto);
		g_free(hostname);
		g_free(user);
		g_free(port_string);
	}
}

static void
load_transport_account_data(EditAccountPageData *data)
{
	if (data->transport_account) {

		gchararray hostname, user, port_string;
		gint port;
		GSList *options, *walk;


		tny_openmoko_account_store_get_account_info	(TNY_OPENMOKO_ACCOUNT_STORE(data->app_data->account_store),
								 tny_account_get_id(data->transport_account),
								 TNY_OPENMOKO_ACCOUNT_HOSTNAME, &hostname,
								 TNY_OPENMOKO_ACCOUNT_PORT, &port,
								 TNY_OPENMOKO_ACCOUNT_USER, &user,
								 TNY_OPENMOKO_ACCOUNT_OPTIONS, &options,
								 -1);

		port_string = g_strdup_printf("%d", port);

		gtk_entry_set_text(GTK_ENTRY(data->out_hostname), hostname);
		gtk_entry_set_text(GTK_ENTRY(data->out_port), port_string);
		gtk_entry_set_text(GTK_ENTRY(data->out_user), user);
		for (walk = options; walk; walk = options->next) {
			if (!strncmp(walk->data, "use_ssl=", 8))
				set_combo_value(GTK_COMBO_BOX(data->out_ssl), encryption, strchr(walk->data, '=') + 1);
			free(walk->data);
		}

		g_slist_free(options);
		g_free(hostname);
		g_free(user);
		g_free(port_string);
	
	}
}

static void
load_account_data(EditAccountPageData *data)
{
	find_transport_account(data);
	find_identities(data);
	load_store_account_data(data);
	load_transport_account_data(data);

	if (data->transport_account)
		tny_openmoko_acount_store_find_folder_async (TNY_OPENMOKO_ACCOUNT_STORE(data->app_data->account_store),
							     data->trasnport_account,
							     TNY_FOLDER_TYPE_SENT,
							     TRUE, /* TODO */
							     (TnyOpenMokoFindFolderCallback) load_account_data_sent_folder,
							     (TnyStatusCallback) status_update,
							     data);
	else
		load_account_data_sent_folder (NULL, FALSE, NULL, NULL, data);
}


static void
load_account_data_sent_folder (TnyOpenMokoAccountStore *self, gboolean cancelled, TnyFolder *folder, GError *err, EditAccountPageData *data)
{
	if (folder) {
		SetFolderData set_folder_data = { folder, data->sent_folder };
		gtk_tree_model_foreach (data->app_data->folder_list,
		                        (GtkTreeModelForeachFunc) set_folder_combo_to,
		                        &set_folder_data);
	}

	if (data->transport_account)
		tny_openmoko_acount_store_find_folder_async (TNY_OPENMOKO_ACCOUNT_STORE(data->app_data->account_store),
							     data->trasnport_account,
							     TNY_FOLDER_TYPE_DRAFTS,
							     TRUE, /* TODO */
							     (TnyOpenMokoFindFolderCallback) load_account_data_drafts_folder,
							     (TnyStatusCallback) status_update,
							     data);
	else
		load_account_data_drafts_folder (NULL, FALSE, NULL, NULL, data);
}

static void
load_account_data_drafts_folder (TnyOpenMokoAccountStore *self, gboolean cancelled, TnyFolder *folder, GError *err, EditAccountPageData *data)
{
	if (folder) {
		SetFolderData set_folder_data = { folder, data->drafts_folder };
		gtk_tree_model_foreach (data->app_data->folder_list,
		                        (GtkTreeModelForeachFunc) set_folder_combo_to,
		                        &set_folder_data);
	}

	if (data->store_account)
		tny_openmoko_acount_store_find_folder_async (TNY_OPENMOKO_ACCOUNT_STORE(data->app_data->account_store),
							     data->store_account,
							     TNY_FOLDER_TYPE_TRASH,
							     TRUE, /* TODO */
							     (TnyOpenMokoFindFolderCallback) load_account_data_trash_folder,
							     (TnyStatusCallback) status_update,
							     data);
	else
		load_account_data_drafts_folder (NULL, FALSE, NULL, NULL, data);
}

static void
load_account_data_trash_folder (TnyOpenMokoAccountStore *self, gboolean cancelled, TnyFolder *folder, GError *err, EditAccountPageData *data)
{
	if (folder) {
		SetFolderData set_folder_data = { folder, data->trash_folder };
		gtk_tree_model_foreach (data->app_data->folder_list,
		                        (GtkTreeModelForeachFunc) set_folder_combo_to,
		                        &set_folder_data);
	}
	gtk_widget_show(page);
}




/* Checking field validity functions */
/* FIXME: The checking code is unbelievably ugly. My checking code always is such.
   Couldn't we just write some checking firmware which gets uploaded in the user's head,
   and checks his input himself? */
enum {
	FIELD_FILLED = 1 << 0,
	FIELD_INVALID = 1 << 1,
	REQUIRED_FIELD_MISSING = 1 << 2
};

enum {
	FIELD_STORE_ACCOUNT =  1 << 0,
	FIELD_TRANSPORT_ACCOUNT = 1 << 1,
	FIELD_NAME = 1 << 2,
	FIELD_PROTO = 1 << 3,
	FIELD_HOSTNAME = 1 << 4,
	FIELD_PORT = 1 << 5,
	FIELD_SENT_FOLDER = 1 << 6,
	FIELD_DRAFTS_FOLDER = 1 << 7,
	FIELD_TRASH_FOLDER = 1 << 8,
	FIELD_IDENTITY = 1 << 9
	
};
	

typedef gboolean (*VFunction) (const gchar *text);

static gint
entry_status(GtkEntry *entry, gboolean required, VFunction validity_function)
{
	gint result = 0;

	const gchar* text = gtk_entry_get_text(entry);
	if (strlen(text)) {
		result |= FIELD_FILLED;
		if (validity_function && !validity_function(text))
			result |= FIELD_INVALID;
	}
	else if (required)
		result |= REQUIRED_FIELD_MISSING;
	return result;
}

static gint
combo_status(GtkComboBox *combo, gboolean required)
{
	gint result;
	if (gtk_combo_box_get_active (combo) != -1)
		result |= FIELD_FILLED;
	else if (required)
		result |= REQUIRED_FIELD_MISSING;
	return result;
}

static
folder_combo_status (GtkComboBox *combo, gboolean required)
{
	GtkTreeIter iter;
	gint result, type;
	if (gtk_combo_box_get_active_iter(data->sent_folder, &iter)) {
		result |= FIELD_FILLED;
		gtk_tree_model_get (data->app_data->folder_list, &iter,
				    TNY_GTK_FOLDER_STORE_TREE_MODEL_TYPE_COLUMN, &type, -1);
		if (type == TNY_FOLDER_TYPE_ROOT)
			result |= FIELD_INVALID;
	}
	else if (required)
		result |= REQUIRED_FIELD_MISSING;
}

gboolean is_valid_port(const gchar *text)
{
	gint port = -1;
	/* FIXME: 143slfjha is a valid port, alright :) */
	if (sscanf(text, "%d", &port) == 0 || port < 0 || port > 65535)
		return FALSE;
	return TRUE;
}

gboolean is_valid_identity(const gchar *text)
{
	return is_address_field_valid(text, TRUE);
}

gboolean is_valid_hostname(const gchar *text)
{

	/* TODO */
	return TRUE;
}



/* Construct a "a, b, c and d" string from n_items items */
/* FIXME: Sucks, localisation-wise and otherwise, but works */
static gchar* generate_displayable_list(gint n_items, ...)
{

	gint i, j;
	va_list ap;
	va_start(ap, attr_type);

	gchar** item_array = g_malloc(n_items * sizeof(const gchar *));
	const gchar* item;
	gchar *result;


	for (i = 0, j = 0; i < n_items; i++) {
		item = va_arg(ap, const gchar*);
		if (item)
			item_array[j++] = item;
	}
	
	if (j == 0)
		result = NULL;
	else if (j == 1)
		result = g_strdup(item_array[0]);
	else {
		item_array[n_items - 2] = g_strdup_printf(_("%s and %s"),
		                                          item_array[n_items - 2], 
		                                          item_array[n_items - 1]);
		item_array[n_items - 1] = NULL;

		result = g_strjoinv (", ", item_array);

		g_free(item_array[n_items - 2]);
	}
	g_free(item_array);
	va_end(ap);
	return result;
}

static gchar* problem_field_list(gint mask) {
	return generate_displayable_list(7,
	                                 mask & FIELD_NAME          ? _("name")          : NULL,
	                                 mask & FIELD_PROTO         ? _("proto")         : NULL,
	                                 mask & FIELD_HOSTNAME      ? _("hostname")      : NULL,
	                                 mask & FIELD_PORT          ? _("port")          : NULL,
	                                 mask & FIELD_SENT_FOLDER   ? _("sent folder")   : NULL,
	                                 mask & FIELD_DRAFTS_FOLDER ? _("drafts folder") : NULL,
	                                 mask & FIELD_TRASH_FOLDER  ? _("trash folder")  : NULL);
}

static gchar* problem_account_list(gint mask) {
	return generate_displayable_list(3,
	                                 mask & FIELD_STORE_ACCOUNT      ? _("incoming account")   : NULL,
	                                 mask & FIELD_TRANSPORT_ACCCOUNT ? _("outgoing account")   : NULL,
	                                 mask & (FIELD_TRANSPORT_ACCCOUNT |  FIELD_STORE_ACCOUNT) == 0    
	                                                                 ? _("extra settings")     : NULL);
}



static gboolean
check_field_validity (EditAccountPageData *data)
{

	gint check, all_checks = 0, empty = 0, required = 0, invalid = 0, result = ACCOUNT_INFO_VALID;
	GSList *walk;

	/* The status of incoming fields. */
	check = entry_status(data->name, TRUE, NULL);
	if (check & REQUIRED_FIELD_MISSING) empty |= FIELD_NAME | FIELD_STORE_ACCOUNT;
	all_checks |= check;

	check = combo_status(data->in_proto, TRUE);
	if (check & REQUIRED_FIELD_MISSING) empty |= FIELD_PROTO | FIELD_STORE_ACCOUNT;
	all_checks |= check;

	check = entry_status(data->in_hostname, TRUE, (VFunction) is_valid_hostname);
	if (check & REQUIRED_FIELD_MISSING) empty |= FIELD_HOSTNAME | FIELD_STORE_ACCOUNT;
	if (check & FIELD_INVALID)  invalid |= FIELD_HOSTNAME | FIELD_STORE_ACCOUNT;
	all_checks |= check;
	
	check = entry_status(data->in_port, FALSE, (VFunction) is_valid_port);
	if (check & FIELD_INVALID) invalid |= FIELD_PORT | FIELD_STORE_ACCOUNT;
	all_checks |= check;

	all_checks |= entry_status(data->in_user, FALSE, NULL);
	all_checks |= combo_status(data->in_ssl, FALSE);

	/* Folder connected to the store account */
	if (data->trash_folder_altered) {
		check = folder_combo_status(data->trash_folder, FALSE);
		if (check & FIELD_INVALID) invalid |= FIELD_TRASH_FOLDER;
		all_checks |= check;
	}

	if (!(all_checks & FIELD_FILLED))
		result |= STORE_ACCOUNT_EMPTY;
	else 
		required |= empty;

	if (((all_checks & FIELD_FILLED) && (all_checks & REQUIRED_DATA_MISSING)) || (all_checks & FIELD_INVALID)
		result |= STORE_ACCOUNT_INVALID;
	
	all_checks = 0;	

	/* The status of outgoing accounts */
	check = entry_status(data->out_hostname, TRUE, (VFunction) is_valid_hostname);
	if (check & REQUIRED_FIELD_MISSING) empty |= FIELD_HOSTNAME | FIELD_TRANSPORT_ACCOUNT;
	if (check & FIELD_INVALID)  invalid |= FIELD_HOSTNAME | FIELD_TRANSPORT_ACCOUNT;
	all_checks |= check;
	
	check = entry_status(data->out_port, FALSE, (VFunction) is_valid_port);
	if (check & FIELD_INVALID) invalid |= FIELD_PORT | FIELD_TRANSPORT_ACCOUNT;
	all_checks |= check;

	all_checks |= entry_status(data->out_user, FALSE, NULL);
	all_checks |= combo_status(data->out_ssl, FALSE);

	/* Identity status */
	for (walk = data->identities; walk; walk = walk->next) {
		check = entry_status(walk->data, FALSE, (VFunction) is_valid_identity);
		if (check & FIELD_INVALID)  invalid |= FIELD_IDENTITY | FIELD_TRANSPORT_ACCOUNT;
		all_checks |= check;
	}



	/* Folder connected to the trasnport account */
	if (sent_folder_altered) {
		check = folder_combo_status(data->sent_folder, FALSE);
		if (check & FIELD_INVALID) invalid |= FIELD_SENT_FOLDER;
		all_checks |= check;
	}

	if (drafts_folder_altered) {
		check = folder_combo_status(data->drafts_folder, FALSE);
		if (check & FIELD_INVALID) invalid |= FIELD_DRAFTS_FOLDER;
		all_checks |= check;
	}

	if (!(all_checks & FIELD_FILLED))
		result |= TRASNPORT_ACCOUNT_EMPTY;
	else 
		required |= empty;

	if (((all_checks & FIELD_FILLED) && (all_checks & REQUIRED_DATA_MISSING)) || (all_checks & FIELD_INVALID)
		result |= TRANSPORT_ACCOUNT_INVALID;
	
	if (result & (STORE_ACCOUNT_INVALID | 
	              STORE_ACCOUNT_EMPTY | 
	              TRANSPORT_ACCOUNT_INVALID) != 0)
	{
		gint response;
		GtkWidget *dialog;
		gchararray invalid_fields = problem_field_list(invalid | required);
		gchararray invalid_accounts = problem_account_list(invalid | required);
		quick_dialog_run(dialog, response, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
		 		 _("The info entered in %s is invalid. Please recheck %s."), 
				 invalid_accounts, invalid_fields);
		g_free(invalid_fields);
		g_free(invalid_accounts);
		
	}
	if (result & TRANSPORT_ACCOUNT_EMPTY) {
		gint response;
		GtkWidget *dialog;
		quick_dialog_run(dialog, dialog_result, GTK_MESSAGE_WARNING, GTK_BUTTONS_YES_NO,
				 _("Are you sure you want to leave the outgoing account empty?"));
		if (response != GTK_RESPONSE_YES)
			result |= TRANSPORT_ACCOUNT_INVALID;
	}
	return result;
}


static void
cb_close_page (GtkToolButton *button, EditAccountPageData *data)
{
	gtk_widget_destroy(data->page);
	g_object_unref(data->store_account);
	g_object_unref(data->transport_account);
	g_slist_free(data->identities);
	g_free(data);
}



static void
cb_save (GtkToolButton *button, EditAccountPageData *data)
{

	gint validity = check_field_validity(data);
	if (validity & (STORE_ACCOUNT_INVALID | 
	                STORE_ACCOUNT_EMPTY | 
	                TRANSPORT_ACCOUNT_INVALID) != 0)
		return;
	
	TnyFolder *folder1 = NULL, *folder2 = NULL;
	GtkTreeModel *model;
	GtkTreeIter iter;

	const gchar *name, *proto, *hostname, *user, *ssl;
	gint port;

	name = gtk_entry_get_text(data->name);
	proto = find_combo_value(data->in_proto, protos);
	hostname = gtk_entry_get_text(data->in_hostname);
	user = gtk_entry_get_text(data->in_user);
	ssl = find_combo_value(data->in_ssl, encryption);
	sscanf(gtk_entry_get_text(data->in_port), "%d", &port);

	if (data->trash_folder_altered) {
		if (gtk_combo_box_get_active_iter(data->trash_folder, &iter)) {
			gtk_tree_model_get (data->app_data->folder_list, &iter,
					    TNY_GTK_FOLDER_STORE_TREE_MODEL_INSTANCE_COLUMN, &folder1, -1);
			if (folder1 && !TNY_IS_FOLDER(folder1)) {
				g_object_unref(folder1);
				folder1 = NULL;
			}
		}
	}

	/* If we have an account object, then we are changing the options,
	   else we're creating a new one */
	if (data->store_account) {
		
		const gchar *old_name, *old_proto, *old_hostname, *old_user;
		gint old_port;
		GSList *options, *walk;
		gboolean options_changed = FALSE;
		
		tny_openmoko_account_store_get_account_info	
			(TNY_OPENMOKO_ACCOUNT_STORE(data->app_data->account_store),
			 tny_account_get_id(data->store_account),
			 TNY_OPENMOKO_ACCOUNT_NAME, &old_name,
			 TNY_OPENMOKO_ACCOUNT_PROTO, &old_proto,
			 TNY_OPENMOKO_ACCOUNT_HOSTNAME, &old_hostname,
			 TNY_OPENMOKO_ACCOUNT_PORT, &old_port,
			 TNY_OPENMOKO_ACCOUNT_USER, &old_user,
			 TNY_OPENMOKO_ACCOUNT_OPTIONS, &options,
			 -1);

		if (!old_name) old_name = g_strdup("");
		if (!old_proto) old_proto = g_strdup("");
		if (!old_hostname) old_hostname = g_strdup("");
		if (!old_user) old_user = g_strdup("");
		
		for (walk = options; walk; walk = walk->next) {
			if (!strncmp(walk->data, "use_ssl=", 8) && strcmp(strchr(walk->data, '=') + 1, ssl))
			{
				options_changed = TRUE;
				g_free(walk->data);
				walk->data = g_strdup_printf("use_ssl=%s", ssl);
				break;
			}
		}

		
		
		tny_openmoko_account_store_update_account
			(TNY_OPENMOKO_ACCOUNT_STORE(data->app_data->account_store),
			 tny_account_get_id(data->store_account),
			 strcmp(name, old_name) 
				? TNY_OPENMOKO_ACCOUNT_NAME 
				: TNY_OPENMOKO_ACCOUNT_SKIP_POINTER, name,
			 strcmp(proto, old_proto) 
				? TNY_OPENMOKO_ACCOUNT_PROTO 
				: TNY_OPENMOKO_ACCOUNT_SKIP_POINTER, proto,
			 strcmp(hostname, old_hostname)
				? TNY_OPENMOKO_ACCOUNT_HOSTNAME
				: TNY_OPENMOKO_ACCOUNT_SKIP_POINTER, hostname,
			 port != old_port 
				? TNY_OPENMOKO_ACCOUNT_PORT
				: TNY_OPENMOKO_ACCOUNT_SKIP_INT, port,
			 strcmp(user, old_user) 
				? TNY_OPENMOKO_ACCOUNT_USER
				: TNY_OPENMOKO_ACCOUNT_SKIP_POINTER, *user ? user : NULL,
			 options_changed 
				? TNY_OPENMOKO_ACCOUNT_OPTIONS
				: TNY_OPENMOKO_ACCOUNT_SKIP_POINTER, options,
			 data->trash_folder_altered
				? TNY_OPENMOKO_ACCOUNT_TRASH_FOLDER
				: TNY_OPENMOKO_ACCOUNT_SKIP_POINTER, folder1,
			 -1);


		g_free(old_name);
		g_free(old_proto);
		g_free(old_hostname);
		g_free(old_user);


	}
	else {
		options = g_slist_append(options, g_strdup_printf("use_ssl=%s", ssl);
		tny_openmoko_account_store_add_account_custom
			(TNY_OPENMOKO_ACCOUNT_STORE(data->app_data->account_store),
			 TNY_OPENMOKO_ACCOUNT_NAME, name,
			 TNY_OPENMOKO_ACCOUNT_PROTO, proto,
			 TNY_OPENMOKO_ACCOUNT_HOSTNAME, hostname, 
			 TNY_OPENMOKO_ACCOUNT_PORT, port,
			 TNY_OPENMOKO_ACCOUNT_USER, *user ? user : NULL,
			 TNY_OPENMOKO_ACCOUNT_OPTIONS, options,
			 TNY_OPENMOKO_ACCOUNT_TRASH_FOLDER, folder1,
			 -1);
	}

	if (folder1) {
		g_object_unref(folder1);
		folder1 = NULL;
	}
	if (options) {
		g_slist_foreach(options, (GFunc) g_free, NULL);
		g_slist_free(options);
		options = NULL;
	}

	/* If there is info about a transport account entered... */
	if (validity & TRANSPORT_ACCOUNT_EMPTY == 0) {
		gchararray transport_name = g_strdup_printf("OUT: %s", name);

		hostname = gtk_entry_get_text(data->out_hostname);
		user = gtk_entry_get_text(data->out_user);
		ssl = find_combo_value(data->out_ssl, encryption);
		sscanf(gtk_entry_get_text(data->out_port), "%d", &port);
	
		if (data->sent_folder_altered) {
			if (gtk_combo_box_get_active_iter(data->sent_folder, &iter)) {
				gtk_tree_model_get (data->app_data->folder_list, &iter,
						    TNY_GTK_FOLDER_STORE_TREE_MODEL_INSTANCE_COLUMN, &folder1, -1);
				if (folder1 && !TNY_IS_FOLDER(folder1)) {
					g_object_unref(folder1);
					folder1 = NULL;
				}
			}
		}
	

		if (data->drafts_folder_altered) {
			if (gtk_combo_box_get_active_iter(data->drafts_folder, &iter)) {
				gtk_tree_model_get (data->app_data->folder_list, &iter,
						    TNY_GTK_FOLDER_STORE_TREE_MODEL_INSTANCE_COLUMN, &folder2, -1);
				if (folder2 && !TNY_IS_FOLDER(folder2)) {
					g_object_unref(folder2);
					folder2 = NULL;
				}
			}
		}

		/* If we have an account object, then we are changing the options,
		   else we're creating a new one */
		if (data->trasnport_account) {
			
			const gchar *old_name, *old_hostname, *old_user;
			gint old_port;
			GSList *options, *walk;
			gboolean options_changed = FALSE;
			
			tny_openmoko_account_store_get_account_info	
				(TNY_OPENMOKO_ACCOUNT_STORE(data->app_data->account_store),
				 tny_account_get_id(data->transport_account),
				 TNY_OPENMOKO_ACCOUNT_NAME, &old_name,
				 TNY_OPENMOKO_ACCOUNT_HOSTNAME, &old_hostname,
				 TNY_OPENMOKO_ACCOUNT_PORT, &old_port,
				 TNY_OPENMOKO_ACCOUNT_USER, &old_user,
				 TNY_OPENMOKO_ACCOUNT_OPTIONS, &options,
				 -1);
	
			if (!old_name) old_name = g_strdup("");
			if (!old_hostname) old_hostname = g_strdup("");
			if (!old_user) old_user = g_strdup("");
			
			for (walk = options; walk; walk = walk->next) {
				if (!strncmp(walk->data, "use_ssl=", 8) && strcmp(strchr(walk->data, '=') + 1, ssl))
				{
					options_changed = TRUE;
					g_free(walk->data);
					walk->data = g_strdup_printf("use_ssl=%s", ssl);
					break;
				}
			}
	
			
			
			tny_openmoko_account_store_update_account
				(TNY_OPENMOKO_ACCOUNT_STORE(data->app_data->account_store),
				 tny_account_get_id(data->transport_account),
				 strcmp(name, old_name) 
					? TNY_OPENMOKO_ACCOUNT_NAME 
					: TNY_OPENMOKO_ACCOUNT_SKIP_POINTER, name,
				 strcmp(hostname, old_hostname)
					? TNY_OPENMOKO_ACCOUNT_HOSTNAME
					: TNY_OPENMOKO_ACCOUNT_SKIP_POINTER, hostname,
				 port != old_port 
					? TNY_OPENMOKO_ACCOUNT_PORT
					: TNY_OPENMOKO_ACCOUNT_SKIP_INT, port,
				 strcmp(user, old_user) 
					? TNY_OPENMOKO_ACCOUNT_USER
					: TNY_OPENMOKO_ACCOUNT_SKIP_POINTER, *user ? user : NULL,
				 options_changed 
					? TNY_OPENMOKO_ACCOUNT_OPTIONS
					: TNY_OPENMOKO_ACCOUNT_SKIP_POINTER, options,
				 data->sent_folder_altered
					? TNY_OPENMOKO_ACCOUNT_TRASH_FOLDER
					: TNY_OPENMOKO_ACCOUNT_SKIP_POINTER, folder1,
				 data->drafts_folder_altered
					? TNY_OPENMOKO_ACCOUNT_TRASH_FOLDER
					: TNY_OPENMOKO_ACCOUNT_SKIP_POINTER, folder2,
				 -1);
	
	
			g_free(old_name);
			g_free(old_hostname);
			g_free(old_user);
	
	
		}
		else {
			options = g_slist_append(options, g_strdup_printf("use_ssl=%s", ssl);
			tny_openmoko_account_store_add_account_custom
				(TNY_OPENMOKO_ACCOUNT_STORE(data->app_data->account_store),
				 TNY_OPENMOKO_ACCOUNT_NAME, name,
				 TNY_OPENMOKO_ACCOUNT_PROTO, "smtp",
				 TNY_OPENMOKO_ACCOUNT_HOSTNAME, hostname, 
				 TNY_OPENMOKO_ACCOUNT_PORT, port,
				 TNY_OPENMOKO_ACCOUNT_USER, *user ? user : NULL,
				 TNY_OPENMOKO_ACCOUNT_OPTIONS, options,
				 TNY_OPENMOKO_ACCOUNT_SENT_FOLDER, folder1,
				 TNY_OPENMOKO_ACCOUNT_DRAFTS_folder, folder2,
				 -1);
		}
	
		if (folder1) {
			g_object_unref(folder1);
			folder1 = NULL;
		}
		if (folder2) {
			g_object_unref(folder1);
			folder1 = NULL;
		}
		if (options) {
			g_slist_foreach(options, (GFunc) g_free, NULL);
			g_slist_free(options);
			options = NULL;
		}

		g_free(transport_name);
		
	}

	




	cb_close_page(NULL, data);
}

GtkWidget*
messages_create_edit_account_page (AppData* app_data, TnyAccount *account, gint page)
{
	GtkWidget *page, *notebook, *subpage, *toolbar, *button, *scroll, *label, *hbox;


	EditAccountPageData *data = g_malloc0(sizeof(EditAccountPageData));
	data->app_data = app_data;

	page = gtk_vbox_new (FALSE, 0);
	data->page = page;

	toolbar = gtk_toolbar_new ();

	toolbar_insert_button (GTK_TOOLBAR(toolbar), gtk_image_new_from_icon_name (GTK_STOCK_SAVE, GTK_ICON_SIZE_LARGE_TOOLBAR), G_CALLBACK(cb_save), data);
	toolbar_insert_button (GTK_TOOLBAR(toolbar), gtk_image_new_from_icon_name (GTK_STOCK_CLOSE, GTK_ICON_SIZE_LARGE_TOOLBAR), G_CALLBACK(cb_close_page), data);

	gtk_widget_show_all (toolbar);

	gtk_box_pack_start (GTK_BOX (page), toolbar, FALSE, TRUE, 0);	

	notebook = gtk_notebook_new ();
	gtk_notebook_set_tab_pos (GTK_NOTEBOOK(notebook), GTK_POS_BOTTOM);
	gtk_widget_show(notebook);

	
	/* 1. Incoming account */
	subpage = gtk_vbox_new(FALSE, 0);

	data->name = moko_hint_entry_new (_("Name"));
	gtk_box_pack_start (GTK_BOX (subpage), data->name, FALSE, TRUE, 0);	


	data->in_proto = gtk_combo_box_new_text();
	combo_fill_in(GTK_COMBO_BOX(data->in_proto), protos);
	gtk_combo_box_set_active(GTK_COMBO_BOX(data->in_proto), 0);
	gtk_box_pack_start (GTK_BOX (subpage), data->in_proto, FALSE, TRUE, 0);

	data->in_ssl = gtk_combo_box_new_text();
	combo_fill_in(GTK_COMBO_BOX(data->in_ssl), encryption);
	gtk_combo_box_set_active(GTK_COMBO_BOX(data->in_ssl), 0);
	gtk_box_pack_start (GTK_BOX (subpage), data->in_ssl, FALSE, TRUE, 0);



	hbox = gtk_hbox_new (FALSE, 0);
	data->in_hostname = moko_hint_entry_new (_("Hostname"));
	gtk_box_pack_start (GTK_BOX (hbox), data->in_hostname, FALSE, TRUE, 0);	

	data->in_port = moko_hint_entry_new (_("Port"));
	gtk_entry_set_max_length (GTK_ENTRY(data->in_port), 6);
	gtk_entry_set_width_chars (GTK_ENTRY(data->in_port), 5);
	gtk_box_pack_start (GTK_BOX (hbox), data->in_port, FALSE, TRUE, 0);	

	gtk_box_pack_start (GTK_BOX (subpage), hbox, FALSE, TRUE, 0);	

	data->in_user = moko_hint_entry_new (_("Username"));
	gtk_box_pack_start (GTK_BOX (subpage), data->in_user, FALSE, TRUE, 0);	

	gtk_widget_show_all(subpage);
	gtk_notebook_append_page (GTK_NOTEBOOK(notebook), subpage, gtk_label_new(_("In"));

	/* 2. Outgoing account */
	subpage = gtk_vbox_new(FALSE, 0);



	data->out_ssl = gtk_combo_box_new_text();
	combo_fill_in(GTK_COMBO_BOX(data->out_ssl), encryption);
	gtk_combo_box_set_active(GTK_COMBO_BOX(data->out_ssl), 0);
	gtk_box_pack_start (GTK_BOX (subpage), data->out_ssl, FALSE, TRUE, 0);



	hbox = gtk_hbox_new (FALSE, 0);
	data->out_hostname = moko_hint_entry_new (_("Hostname"));
	gtk_box_pack_start (GTK_BOX (hbox), data->out_hostname, TRUE, TRUE, 0);	

	data->out_port = moko_hint_entry_new (_("Port"));
	gtk_entry_set_max_length (GTK_ENTRY(data->out_port), 6);
	gtk_entry_set_width_chars (GTK_ENTRY(data->out_port), 5);
	gtk_box_pack_start (GTK_BOX (hbox), data->out_port, FALSE, TRUE, 0);	

	gtk_box_pack_start (GTK_BOX (subpage), hbox, FALSE, TRUE, 0);	

	data->out_user = moko_hint_entry_new (_("Username"));
	gtk_box_pack_start (GTK_BOX (subpage), data->out_user, FALSE, TRUE, 0);	
	

	data->identity_box = GTK_BOX(subpage);

	gtk_widget_show_all(subpage);
	gtk_notebook_append_page (GTK_NOTEBOOK(notebook), subpage, gtk_label_new(_("Out"));


	/* 3. Extra account info */
	if (account) {
		subpage = gtk_vbox_new(FALSE, 0);

		
		gtk_box_pack_start (GTK_BOX(subpage), gtk_label_new(_("Sent folder"), FALSE, TRUE, 0);
		hbox = gtk_hbox_new(FALSE, 0):

		data->sent_folder = gtk_combo_box_new_with_model (app_data->folder_list);
		g_signal_connect_swapped(G_OBJECT(data->sent_folder), "changed",
		                         G_CALLBACK(set_to_true), &(data->sent_folder_changed));
		gtk_box_pack_start (GTK_BOX (hbox), data->sent_folder, TRUE, TRUE, 0);

		button = gtk_button_new_with_label("x");
		g_signal_connect_swapped(G_OBJECT(button), "clicked",
		                         G_CALLBACK(unset_combo_box), GTK_COMBO_BOX(data->sent_folder));
		gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, TRUE, 0);
		gtk_box_pack_start (GTK_BOX (subpage), hbox, FALSE, TRUE, 0);

	

		gtk_box_pack_start (GTK_BOX(subpage), gtk_label_new(_("Drafts folder"), FALSE, TRUE, 0);
		hbox = gtk_hbox_new(FALSE, 0):

		data->drafts_folder = gtk_combo_box_new_with_model (app_data->folder_list);
		g_signal_connect_swapped(G_OBJECT(data->drafts_folder), "changed",
		                         G_CALLBACK(set_to_true), &(data->drafts_folder_changed));
		gtk_box_pack_start (GTK_BOX (hbox), data->drafts_folder, TRUE, TRUE, 0);

		button = gtk_button_new_with_label("x");
		g_signal_connect_swapped(G_OBJECT(button), "clicked",
		                         G_CALLBACK(unset_combo_box), GTK_COMBO_BOX(data->drafts_folder));
		gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, TRUE, 0);
		gtk_box_pack_start (GTK_BOX (subpage), hbox, FALSE, TRUE, 0);



		gtk_box_pack_start (GTK_BOX(subpage), gtk_label_new(_("Trash folder"), FALSE, TRUE, 0);
		hbox = gtk_hbox_new(FALSE, 0):

		data->trash_folder = gtk_combo_box_new_with_model (app_data->folder_list);
		g_signal_connect_swapped(G_OBJECT(data->trash_folder), "changed",
		                         G_CALLBACK(set_to_true), &(data->trash_folder_changed));
		gtk_box_pack_start (GTK_BOX (hbox), data->trash_folder, TRUE, TRUE, 0);

		button = gtk_button_new_with_label("x");
		g_signal_connect_swapped(G_OBJECT(button), "clicked",
		                         G_CALLBACK(unset_combo_box), GTK_COMBO_BOX(data->trash_folder));
		gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, TRUE, 0);
		gtk_box_pack_start (GTK_BOX (subpage), hbox, FALSE, TRUE, 0);

	
	
		gtk_widget_show_all(subpage);
		gtk_notebook_append_page (GTK_NOTEBOOK(notebook), subpage, gtk_label_new(_("Extra"));

		load_account_data(data);
	}
	else {
		
		add_identity(data, NULL);
		add_identity(data, NULL);
		add_identity(data, NULL);
		gtk_widget_show (page);
	}

#if 0
	gtk_notebook_set_current_page (GTK_NOTEBOOK(notebook), page);
#endif

	gtk_box_pack_start (GTK_BOX (page), notebook, TRUE, TRUE, 0);	

	data->store_account = account;
	get_account_data(data);
	
	return page;

	
}

