/*
 * Mail for OpenMoko
 * Copyright (C) 2007 Milko Krachounov <exabyte@3mhz.net>
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include <gtk/gtk.h>
#include <glib/gi18n.h>

#include <moko-finger-scroll.h>
#include <moko-stock.h>

#include <glib/gstdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <strings.h>

#include <tny-msg.h>
#include <tny-header.h>
#include <tny-mime-part.h>
#include <tny-fs-stream.h>
#include <tny-platform-factory.h>
/* FIXME: I break the concept of the platform factory here. */
#include <tny-openmoko-msg-view.h>

#include "mokofieldlabel.h"
#include "mail-defines.h"
#include "mail-common.h"
#include "mail-access.h"
#include "mail-list.h"
#include "mail-write.h"
#include "mail-read.h"


typedef struct {
	AppData *app_data;

	TnyFolder *folder;
	TnyHeader *header;
	TnyMsg *message;

	GtkWidget *message_view;

	GtkWidget *page;
} MessagesReadPageData;





static void
cb_attachment_save(GtkWidget *aview, TnyMimePart *attachment, MessagesReadPageData *data)
{
	

	GtkWidget *dialog;
	gint dialog_result;

	gchararray attachment_file;

	TnyStream *stream;
	int fd;

	/* TODO: Implement folder chooser */
	const gchar *path = "/media/card/";
	const gchar *filename = tny_mime_part_get_filename (attachment);
	
	attachment_file = g_build_path("/", path, filename, NULL);

	dialog = gtk_message_dialog_new(0, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO, _("Do you want to save the attachment %s in %s?"), filename, path);
	gtk_window_set_title(GTK_WINDOW(dialog), " ");

	dialog_result = gtk_dialog_run(GTK_DIALOG(dialog));
	gtk_widget_destroy(dialog);

	if (dialog_result == GTK_RESPONSE_YES) {
		fd = g_creat (attachment_file, 0644);
		if (fd == -1) {
			dialog = gtk_message_dialog_new(0, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, _("Saving the attachment failed."));
			gtk_dialog_run(GTK_DIALOG(dialog));
			gtk_widget_destroy(dialog);
		}
		else {
			stream = tny_fs_stream_new (fd);
			tny_mime_part_decode_to_stream (attachment, stream);
			g_object_unref(stream);
			close(fd);
		}
	}
	g_free(attachment_file);

}


static void
cb_attachment_delete(GtkWidget *aview, TnyMimePart *attachment, MessagesReadPageData *data)
{
	
	GtkWidget *dialog;
	gint dialog_result;

	const gchar *filename = tny_mime_part_get_filename (attachment);
	
	dialog = gtk_message_dialog_new(0, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO, _("Do you want to remove the attachment %s?"), filename);
	gtk_window_set_title(GTK_WINDOW(dialog), " ");

	dialog_result = gtk_dialog_run(GTK_DIALOG(dialog));
	gtk_widget_destroy(dialog);

	if (dialog_result == GTK_RESPONSE_YES) {
		tny_mime_part_del_part(TNY_MIME_PART(data->message), attachment);
	}

}


static void
cb_message_close (GtkToolButton *button, MessagesReadPageData *data)
{

	gtk_widget_destroy(data->page);

	if (data->folder)
		g_object_unref(G_OBJECT(data->folder));
	if (data->header)
		g_object_unref(G_OBJECT(data->header));
	if (data->message)
		g_object_unref(G_OBJECT(data->message));
	g_free(data);
}


static void
message_display (TnyFolder *folder, gboolean canceled, TnyMsg *message,
                 GError *err, MessagesReadPageData *data)
{

	GtkWidget *mview, *aview, *dialog;
	TnyHeaderFlags flags;
	gint result;

	if (err) {
		quick_dialog_run(dialog, result, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
				 _("Could not open the message: %s\n"), err->message);
		cb_message_close(NULL, data);
		return;
	}

	data->message = g_object_ref(message);
	
	/* TODO: I don't know whether we get a message on cancel? A partially displayed message
                 might be OK, in some cases. Consider. */
	if (!canceled && message) {
		mview = GTK_WIDGET(tny_platform_factory_new_msg_view(data->app_data->platfact));
		flags = tny_header_get_flags (data->header);
		if (!(flags & TNY_HEADER_FLAG_SEEN)) {
			tny_header_set_flag (data->header, TNY_HEADER_FLAG_SEEN);
			/* TODO: Syncing every time might not be the best for a phone */
			tny_folder_sync_async (folder, FALSE, NULL,
				       (TnyStatusCallback) status_update, data->app_data);
		}
		tny_msg_view_set_msg(TNY_MSG_VIEW(mview), message);
		gtk_widget_show(mview);
		gtk_box_pack_start (GTK_BOX (data->message_view), mview, TRUE, TRUE, 0);

		aview = tny_openmoko_msg_view_get_attachment_view(TNY_OPENMOKO_MSG_VIEW(mview));
		g_signal_connect(G_OBJECT(aview), "delete-request", G_CALLBACK(cb_attachment_delete), data);
		g_signal_connect(G_OBJECT(aview), "save-request", G_CALLBACK(cb_attachment_save), data);



		gtk_widget_show (data->page);
		
		
	}
	else {
		cb_message_close(NULL, data);
	}

}


static void
cb_message_reply_all (GtkWidget *widget, MessagesReadPageData *data)
{
	gint page_num;
	GtkWidget *page;
	page = messages_create_write_email_page(data->app_data, MESSAGE_WRITE_REPLY_ALL, data->header, data->message);
	page_num = notebook_add_page_with_icon (data->app_data->notebook, page, MOKO_STOCK_MAIL_NEW, 0);
	gtk_notebook_set_current_page (data->app_data->notebook, page_num);
}


static void
cb_message_reply_sender (GtkWidget *widget, MessagesReadPageData *data)
{

	gint page_num;
	GtkWidget *page;
	page = messages_create_write_email_page(data->app_data, MESSAGE_WRITE_REPLY_SENDER, data->header, data->message);
	page_num = notebook_add_page_with_icon (data->app_data->notebook, page, MOKO_STOCK_MAIL_NEW, 0);
	gtk_notebook_set_current_page (data->app_data->notebook, page_num);
}

static void
cb_message_forward_inline (GtkWidget *widget, MessagesReadPageData *data)
{
	gint page_num;
	GtkWidget *page;
	page = messages_create_write_email_page(data->app_data, MESSAGE_WRITE_FORWARD_INLINE, data->header, data->message);
	page_num = notebook_add_page_with_icon (data->app_data->notebook, page, MOKO_STOCK_MAIL_NEW, 0);
	gtk_notebook_set_current_page (data->app_data->notebook, page_num);
}

static void
cb_message_forward_attachment (GtkWidget *widget, MessagesReadPageData *data)
{

}

static void
cb_message_delete (GtkToolButton *button, MessagesReadPageData *data)
{
	tny_header_set_flag (data->header, TNY_HEADER_FLAG_DELETED);
	tny_folder_sync_async (data->folder, FALSE, NULL,
			       (TnyStatusCallback) status_update, data->app_data);
	cb_message_close(button, data);
}

static void
cb_message_destroy (GtkToolButton *button, MessagesReadPageData *data)
{

	GtkWidget *dialog;
	gint dialog_result;
	GError *err;

	quick_dialog_run(dialog, dialog_result, GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO,
			 _("Delete this message?"));

	if (dialog_result == GTK_RESPONSE_YES) {
		
		tny_folder_remove_msg(data->folder, data->header, &err);
		if (err) {
			quick_dialog_run(dialog, dialog_result, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
					 _("Deletion failed: %s"), err->message);
			g_error_free(err);
		}
		tny_folder_sync_async (data->folder, FALSE, NULL, 
				       (TnyStatusCallback) status_update, data->app_data);	
		cb_message_close(button, data);
	}
}

static void
cb_status_read_page (GObject *sender, TnyStatus *status, MessagesReadPageData *data)
{
	status_update(sender, status, data->app_data);
}

static GtkWidget*
get_menu_message_reply (MessagesReadPageData *data)
{
	GtkWidget *menu, *item;
	menu = gtk_menu_new();

	item = gtk_menu_item_new_with_label (_("Reply to Sender"));
	g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(cb_message_reply_sender), data);
	gtk_menu_shell_append (GTK_MENU_SHELL(menu), item);

	item = gtk_menu_item_new_with_label (_("Reply to All"));
	g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(cb_message_reply_all), data);
	gtk_menu_shell_append (GTK_MENU_SHELL(menu), item);

	gtk_widget_show_all(menu);
	return menu;
}

static GtkWidget*
get_menu_message_forward (MessagesReadPageData *data)
{
	GtkWidget *menu, *item;
	menu = gtk_menu_new();

	item = gtk_menu_item_new_with_label (_("Forward as Inline"));
	g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(cb_message_forward_inline), data);
	gtk_menu_shell_append (GTK_MENU_SHELL(menu), item);

	item = gtk_menu_item_new_with_label (_("Forward as Attachment"));
	g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(cb_message_forward_attachment), data);
	gtk_menu_shell_append (GTK_MENU_SHELL(menu), item);

	gtk_widget_show_all(menu);
	return menu;
}

static GtkWidget*
get_menu_message_delete (MessagesReadPageData *data)
{
	GtkWidget *menu, *item;
	menu = gtk_menu_new();

	item = gtk_menu_item_new_with_label (_("Delete message"));
	g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(cb_message_delete), data);
	gtk_menu_shell_append (GTK_MENU_SHELL(menu), item);

	item = gtk_menu_item_new_with_label (_("Destroy message"));
	g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(cb_message_destroy), data);
	gtk_menu_shell_append (GTK_MENU_SHELL(menu), item);

	gtk_widget_show_all(menu);
	return menu;
}

GtkWidget*
messages_create_read_page (AppData* app_data, TnyFolder *folder, TnyHeader *header)
{
	GtkWidget *page, *toolbar, *viewport, *scroll, *message_view;

	MessagesReadPageData *data = g_malloc0(sizeof(MessagesReadPageData));
		
	page = gtk_vbox_new (FALSE, 0);

	/* We create the toolbar */
	toolbar = gtk_toolbar_new ();

	toolbar_insert_button (GTK_TOOLBAR(toolbar), gtk_image_new_from_icon_name (GTK_STOCK_CLOSE, GTK_ICON_SIZE_LARGE_TOOLBAR), G_CALLBACK(cb_message_close), data);
	toolbar_insert_menu_button (GTK_TOOLBAR(toolbar), gtk_image_new_from_icon_name (MOKO_STOCK_MAIL_REPLY_SENDER, GTK_ICON_SIZE_LARGE_TOOLBAR), G_CALLBACK(cb_message_reply_sender), data, get_menu_message_reply(data));
	toolbar_insert_menu_button (GTK_TOOLBAR(toolbar), gtk_image_new_from_icon_name (MOKO_STOCK_MAIL_FORWARD, GTK_ICON_SIZE_LARGE_TOOLBAR), G_CALLBACK(cb_message_forward_inline), data, get_menu_message_forward(data));
	toolbar_insert_menu_button (GTK_TOOLBAR(toolbar), gtk_image_new_from_icon_name (MOKO_STOCK_MAIL_DELETE, GTK_ICON_SIZE_LARGE_TOOLBAR), G_CALLBACK(cb_message_delete), data, get_menu_message_delete(data));

	gtk_widget_show_all (toolbar);
	gtk_box_pack_start (GTK_BOX (page), toolbar, FALSE, TRUE, 0);
	
	message_view = gtk_vbox_new (FALSE, 0);
	viewport = gtk_viewport_new(NULL, NULL);
	gtk_viewport_set_shadow_type (GTK_VIEWPORT (viewport), GTK_SHADOW_NONE);

	scroll = moko_finger_scroll_new();
	gtk_container_add (GTK_CONTAINER (viewport), message_view);	
	gtk_container_add (GTK_CONTAINER (scroll), viewport);
	gtk_widget_show_all(scroll);
	gtk_box_pack_end (GTK_BOX (page), scroll, TRUE, TRUE, 0);



	data->folder = g_object_ref(folder);
	data->header = g_object_ref(header);
	data->message = NULL;

	data->message_view = message_view;
	
	data->app_data = app_data;
	data->page = page;

	tny_folder_get_msg_async (folder, header, (TnyGetMsgCallback)message_display,
				  (TnyStatusCallback) cb_status_read_page, data);

	return page;

}
